@@ -94,13 +94,10 @@ static void Audio_CallBack(void *userdata, Uint8 *stream, int len)
 			*pBuffer++ = MixBuffer[(CompleteSndBufIdx + i) % MIXBUFFER_SIZE][0];
 			*pBuffer++ = MixBuffer[(CompleteSndBufIdx + i) % MIXBUFFER_SIZE][1];
 		}
-		/* If the buffer is filled more than 50%, mirror sample buffer to fake the
-		 * missing samples */
-		if (nGeneratedSamples >= len/2)
-		{
-			int remaining = len - nGeneratedSamples;
-			memcpy(pBuffer, stream+(nGeneratedSamples-remaining)*4, remaining*4);
-		}
+		/* Clear rest of the buffer to ensure we don't play random bytes instead */
+		/* of missing samples */
+		memset(pBuffer, 0, (len - nGeneratedSamples) * 4);
+
 		CompleteSndBufIdx += nGeneratedSamples;
 		nGeneratedSamples = 0;
 		
@@ -159,9 +156,8 @@ void Audio_Init(void)
 		int samples = (desiredAudioSpec.freq / 1000) * SdlAudioBufferSize;
 		int power2 = 1;
 		while ( power2 < samples )		/* compute the power of 2 just above samples */
-                        power2 *= 2;
+			power2 *= 2;
 
-//fprintf ( stderr , "samples %d power %d\n" , samples , power2 );
 		desiredAudioSpec.samples = power2;	/* number of samples corresponding to the requested SdlAudioBufferSize */
 	}
 
@@ -175,8 +171,12 @@ void Audio_Init(void)
 		return;
 	}
 
+#if WITH_SDL2						/* SDL2 does not set desiredAudioSpec.size anymore */
+	SoundBufferSize = desiredAudioSpec.samples;
+#else
 	SoundBufferSize = desiredAudioSpec.size;	/* May be different than the requested one! */
 	SoundBufferSize /= 4;				/* bytes -> samples (16 bit signed stereo -> 4 bytes per sample) */
+#endif
 	if (SoundBufferSize > MIXBUFFER_SIZE/2)
 	{
 		fprintf(stderr, "Warning: Soundbuffer size is too big!\n");
@@ -239,14 +239,14 @@ void Audio_SetOutputAudioFreq(int nNewFrequency)
 		/* Set new frequency */
 		nAudioFrequency = nNewFrequency;
 
-		if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+		if (Config_IsMachineFalcon())
 		{
 			/* Compute Ratio between host computer sound frequency and Hatari's sound frequency. */
 			Crossbar_Compute_Ratio();
 		}
-		else if (ConfigureParams.System.nMachineType != MACHINE_ST)
+		else if (!Config_IsMachineST())
 		{
-			/* Adapt LMC filters to this new frequency */			
+			/* Adapt LMC filters to this new frequency */
 			DmaSnd_Init_Bass_and_Treble_Tables();
 		}
 
@@ -258,16 +258,8 @@ void Audio_SetOutputAudioFreq(int nNewFrequency)
 		}
 	}
 
-	if ((ConfigureParams.System.nMachineType == MACHINE_ST) &&
-		(nAudioFrequency >= 40000))
-	{
-		/* Apply YM2149 C10 filter. */
-		UseLowPassFilter = true;
-	}
-	else
-	{
-		UseLowPassFilter = false;
-	}
+	/* Apply YM2149 C10 filter? */
+	UseLowPassFilter = Config_IsMachineST() && nAudioFrequency >= 40000;
 }
 
 
