@@ -365,11 +365,11 @@ static bool	Avi_RecordVideoStream_BMP ( RECORD_AVI_PARAMS *pAviParams )
 {
 	AVI_CHUNK	Chunk;
 	int		SizeImage;
-	Uint8		LineBuf[ 3 * pAviParams->Width ];			/* temp buffer to convert to 24-bit BGR format */
 	Uint8		*pBitmapIn , *pBitmapOut;
 	int		y;
 	int		NeedLock;
-	
+	Uint8		*LineBuf = alloca(3 * pAviParams->Width);		/* temp buffer to convert to 24-bit BGR format */
+
 	SizeImage = Avi_GetBmpSize ( pAviParams->Width , pAviParams->Height , pAviParams->BitCount );
 
 	/* Write the video frame header */
@@ -398,6 +398,8 @@ static bool	Avi_RecordVideoStream_BMP ( RECORD_AVI_PARAMS *pAviParams )
 		if ( NeedLock )
 			SDL_LockSurface ( pAviParams->Surface );
 
+		pBitmapIn -= pAviParams->Surface->pitch;			/* go from bottom to top */
+
 		pBitmapOut = LineBuf;
 		switch ( pAviParams->Surface->format->BytesPerPixel ) {
 			case 1 :	PixelConvert_8to24Bits_BGR(LineBuf, pBitmapIn, pAviParams->Width, pAviParams->Surface->format->palette->colors);
@@ -419,8 +421,6 @@ static bool	Avi_RecordVideoStream_BMP ( RECORD_AVI_PARAMS *pAviParams )
 			Log_AlertDlg ( LOG_ERROR, "AVI recording : failed to write bmp video frame" );
 			return false;
 		}
-
-		pBitmapIn -= pAviParams->Surface->pitch;			/* go from bottom to top */
 	}
 
 	return true;
