@@ -135,23 +135,32 @@ static BLITTER_OP_FUNC Blitter_ComputeLOP;
 
 /*-----------------------------------------------------------------------*/
 /**
- * Count blitter cycles
+ * Count blitter cycles (this assumes blitter and CPU runs at the same freq)
  */
 
 static void Blitter_AddCycles(int cycles)
 {
-	int all_cycles = cycles + nWaitStateCycles;
+	int all_cycles = cycles + WaitStateCycles;
 
 	BlitterVars.op_cycles += all_cycles;
 
+#ifdef OLD_CPU_SHIFT
 	nCyclesMainCounter += all_cycles >> nCpuFreqShift;
 	CyclesGlobalClockCounter += all_cycles >> nCpuFreqShift;
-	nWaitStateCycles = 0;
+#else
+	nCyclesMainCounter += all_cycles;
+	CyclesGlobalClockCounter += all_cycles;
+#endif
+	WaitStateCycles = 0;
 }
 
 static void Blitter_FlushCycles(void)
 {
+#ifdef OLD_CPU_SHIFT
+	int op_cycles = INT_CONVERT_TO_INTERNAL(BlitterVars.op_cycles >> nCpuFreqShift, INT_CPU_CYCLE);
+#else
 	int op_cycles = INT_CONVERT_TO_INTERNAL(BlitterVars.op_cycles, INT_CPU_CYCLE);
+#endif
 
 	BlitterVars.pass_cycles += BlitterVars.op_cycles;
 	BlitterVars.op_cycles = 0;
@@ -180,6 +189,7 @@ static void Blitter_FlushCycles(void)
  *  - t+0 : CPU can still run during 4 cycles and access bus
  *  - t+4 : bus arbitration takes 4 cycles (no access for cpu and blitter during this time)
  *  - t+8 : blitter owns the bus and starts tranferring data
+ * (in case of MegaSTE bus arbitration takes 8 cycles instead of 4)
  *
  * When blitter stops owning the bus in favor of the cpu, this seems to always take 4 cycles
  */
@@ -190,19 +200,22 @@ static void Blitter_BusArbitration ( int RequestBusMode )
 	if ( RequestBusMode == BUS_MODE_BLITTER )	/* Bus is requested by the blitter */
 	{
 //fprintf ( stderr , "blitter start pc %x %x\n" , M68000_GetPC() , M68000_InstrPC );
-		cycles = 4;				/* Default case : take 4 cycles when going from cpu to blitter */
+		if ( ConfigureParams.System.nMachineType == MACHINE_MEGA_STE )
+			cycles = 8;			/* MegaSTE blitter needs 4 extra cycles when requesting the bus */
+		else
+			cycles = 4;			/* Default case : take 4 cycles when going from cpu to blitter */
 
 		/* Different timing for some specific cases */
 
 		/* 'Relapse - Graphix Sound 2' by Cybernetics (overscan plasma using blitter) */
 		/* $e764 : move.b  d5,(a4) + dbra d1,$fff2 : 4 cycles of the dbra can be executed while blitter starts */
 		if ( STMemory_ReadLong ( M68000_InstrPC ) == 0x188551c9 )	/* PC = E764 */
-			cycles = 4-4;			/* 4 cycles less than default case */
+			cycles -= 4;			/* 4 cycles less than default case */
 	}
 
 	else						/* Bus is requested by the cpu */
 	{
-		cycles = 4;				/* Always 4 cycles ? */
+		cycles = 4;				/* Always 4 cycles (even for MegaSTE) */
 	}
 
 	/* Add arbitration cycles and update BusMode */
@@ -238,6 +251,7 @@ static Uint16 Blitter_ReadWord(Uint32 addr)
 static void Blitter_WriteWord(Uint32 addr, Uint16 value)
 {
 	/* Call put_word only if the address doesn't point to a bus error region */
+	/* (also see SysMem_wput for addr < 0x8) */
 	if ( STMemory_CheckRegionBusError ( addr ) == false )
 		put_word ( addr , (Uint32)(value) );
 //fprintf ( stderr , "write %x %x %x\n" , addr , value , STMemory_CheckRegionBusError(addr) );
@@ -620,7 +634,11 @@ static void Blitter_Start(void)
 	else
 	{
 		/* Continue blitting later */
+#ifdef OLD_CPU_SHIFT
 		CycInt_AddRelativeInterrupt(NONHOG_CYCLES, INT_CPU_CYCLE, INTERRUPT_BLITTER);
+#else
+		CycInt_AddRelativeInterrupt(NONHOG_CYCLES, INT_CPU_CYCLE, INTERRUPT_BLITTER);
+#endif
 	}
 }
 
@@ -1001,9 +1019,14 @@ void Blitter_Control_WriteByte(void)
 		}
 		else
 		{
-			/* Start blitting after some CPU cycles */
-			CycInt_AddRelativeInterrupt((CurrentInstrCycles+nWaitStateCycles)>>nCpuFreqShift,
+			/* Start blitting after the end of current instruction */
+#ifdef OLD_CPU_SHIFT
+			CycInt_AddRelativeInterrupt((CurrentInstrCycles+WaitStateCycles)>>nCpuFreqShift,
 							 INT_CPU_CYCLE, INTERRUPT_BLITTER);
+#else
+			CycInt_AddRelativeInterrupt( CurrentInstrCycles+WaitStateCycles, INT_CPU_CYCLE, INTERRUPT_BLITTER);
+			/* [NP] TODO : use CycInt_AddRelativeInterrupt(0) instead to get an immediate interrupt */
+#endif
 		}
 	}
 }
