@@ -66,8 +66,8 @@ uae_u32 mmu040_move16[4];
 
 static void mmu_dump_ttr(const TCHAR * label, uae_u32 ttr)
 {
-#if MMUDEBUG > 0			// ifdef WINUAE_FOR_HATARI
 	DUNUSED(label);
+#if MMUDEBUG > 0
 	uae_u32 from_addr, to_addr;
 
 	from_addr = ttr & MMU_TTR_LOGICAL_BASE;
@@ -297,7 +297,7 @@ static void mmu_dump_table(const char * label, uaecptr root_ptr)
 #endif
 
 /* {{{ mmu_dump_atc */
-void mmu_dump_atc(void)
+static void mmu_dump_atc(void)
 {
 
 }
@@ -738,23 +738,21 @@ uae_u32 REGPARAM2 mmu_get_ilong_unaligned(uaecptr addr)
 {
 	uae_u32 res;
 
-	if (likely(!(addr & 1))) {
-		res = (uae_u32)mmu_get_iword(addr, sz_long) << 16;
-		SAVE_EXCEPTION;
-		TRY(prb) {
-			res |= mmu_get_iword(addr + 2, sz_long);
-			RESTORE_EXCEPTION;
-		}
-		CATCH(prb) {
-			RESTORE_EXCEPTION;
-			misalignednotfirst(addr);
-			THROW_AGAIN(prb);
-		} ENDTRY
+	res = (uae_u32)mmu_get_iword(addr, sz_long) << 16;
+	SAVE_EXCEPTION;
+	TRY(prb) {
+		res |= mmu_get_iword(addr + 2, sz_long);
+		RESTORE_EXCEPTION;
 	}
+	CATCH(prb) {
+		RESTORE_EXCEPTION;
+		misalignednotfirst(addr);
+		THROW_AGAIN(prb);
+	} ENDTRY
 	return res;
 }
 
-uae_u16 REGPARAM2 mmu_get_lrmw_word_unaligned(uaecptr addr)
+static uae_u16 REGPARAM2 mmu_get_lrmw_word_unaligned(uaecptr addr)
 {
 	uae_u16 res;
 
@@ -772,7 +770,7 @@ uae_u16 REGPARAM2 mmu_get_lrmw_word_unaligned(uaecptr addr)
 	return res;
 }
 
-uae_u32 REGPARAM2 mmu_get_lrmw_long_unaligned(uaecptr addr)
+static uae_u32 REGPARAM2 mmu_get_lrmw_long_unaligned(uaecptr addr)
 {
 	uae_u32 res;
 
