@@ -24,8 +24,10 @@
  */
 
 
-#ifndef CPUMMU_H
-#define CPUMMU_H
+#ifndef UAE_CPUMMU_H
+#define UAE_CPUMMU_H
+
+#include "uae/types.h"
 
 #include "mmu_common.h"
 
@@ -107,7 +109,6 @@ struct log_addr8 {
 #define MMU_TEST_FORCE_TABLE_SEARCH		4
 #define MMU_TEST_NO_BUSERR				8
 
-extern void mmu_dump_atc(void);
 extern void mmu_dump_tables(void);
 
 #define MMU_TTR_LOGICAL_BASE				0xff000000
@@ -323,8 +324,6 @@ extern void REGPARAM3 mmu060_put_rmw_bitfield (uae_u32 dst, uae_u32 bdata[2], ua
 
 extern uae_u16 REGPARAM3 mmu_get_word_unaligned(uaecptr addr, bool data, bool rmw) REGPARAM;
 extern uae_u32 REGPARAM3 mmu_get_long_unaligned(uaecptr addr, bool data, bool rmw) REGPARAM;
-extern uae_u16 REGPARAM2 mmu_get_lrmw_word_unaligned(uaecptr addr) REGPARAM;
-extern uae_u32 REGPARAM2 mmu_get_lrmw_long_unaligned(uaecptr addr) REGPARAM;
 
 extern uae_u32 REGPARAM3 mmu_get_ilong_unaligned(uaecptr addr) REGPARAM;
 
@@ -924,4 +923,4 @@ extern void m68k_do_rts_mmu060 (void);
 extern void m68k_do_rte_mmu060 (uaecptr a7);
 extern void m68k_do_bsr_mmu060 (uaecptr oldpc, uae_s32 offset);
 
-#endif /* CPUMMU_H */
+#endif /* UAE_CPUMMU_H */
