@@ -1,5 +1,7 @@
-#ifndef CPUMMU030_H
-#define CPUMMU030_H
+#ifndef UAE_CPUMMU030_H
+#define UAE_CPUMMU030_H
+
+#include "uae/types.h"
 
 #include "mmu_common.h"
 
@@ -67,10 +69,6 @@ void mmu030_put_byte_atc(uaecptr addr, uae_u8 val, int l, uae_u32 fc);
 uae_u32 mmu030_get_long_atc(uaecptr addr, int l, uae_u32 fc);
 uae_u16 mmu030_get_word_atc(uaecptr addr, int l, uae_u32 fc);
 uae_u8 mmu030_get_byte_atc(uaecptr addr, int l, uae_u32 fc);
-#ifdef WINUAE_FOR_HATARI
-uae_u32 mmu030_get_ilong_atc(uaecptr addr, int l, uae_u32 fc);
-uae_u16 mmu030_get_iword_atc(uaecptr addr, int l, uae_u32 fc);
-#endif
 
 void mmu030_put_atc_generic(uaecptr addr, uae_u32 val, int l, uae_u32 fc, int size, int flags);
 uae_u32 mmu030_get_atc_generic(uaecptr addr, int l, uae_u32 fc, int size, int flags, bool checkwrite);
@@ -474,4 +472,4 @@ extern void m68k_do_rte_mmu030 (uaecptr a7);
 extern void flush_mmu030 (uaecptr, int);
 extern void m68k_do_bsr_mmu030 (uaecptr oldpc, uae_s32 offset);
 
-#endif
+#endif /* UAE_CPUMMU030_H */
