@@ -123,7 +123,13 @@ uae_u32 wait_cpu_cycle_read (uaecptr addr, int mode)
 	SETIFCHIP
 
 #else						/* WINUAE_FOR_HATARI */
-	x_do_cycles_pre (CYCLE_UNIT);
+//	fprintf ( stderr , "mem read ce %x %d %lu %lu\n" , addr , mode ,currcycle / cpucycleunit , currcycle );
+	if ( ( ( CyclesGlobalClockCounter + currcycle*2/CYCLE_UNIT ) & 3 ) == 2 )
+	{
+//		fprintf ( stderr , "mem wait read %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+		x_do_cycles (2*cpucycleunit);
+//		fprintf ( stderr , "mem wait read after %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+	}
 
 	if (mode < 0)
 		v = get_long (addr);
@@ -132,7 +138,7 @@ uae_u32 wait_cpu_cycle_read (uaecptr addr, int mode)
 	else if (mode == 0)
 		v = get_byte (addr);
 
-	x_do_cycles_post (CYCLE_UNIT, v);
+	x_do_cycles_post (2*CYCLE_UNIT, v);
 #endif						/* WINUAE_FOR_HATARI */
 
 	return v;
@@ -232,7 +238,13 @@ void wait_cpu_cycle_write (uaecptr addr, int mode, uae_u32 v)
 	SETIFCHIP
 
 #else						/* WINUAE_FOR_HATARI */
-	x_do_cycles_pre (CYCLE_UNIT);
+//	fprintf ( stderr , "mem write ce %x %d %lu %lu\n" , addr , mode ,currcycle / cpucycleunit , currcycle );
+	if ( ( ( CyclesGlobalClockCounter + currcycle*2/CYCLE_UNIT ) & 3 ) == 2 )
+	{
+//		fprintf ( stderr , "mem wait write %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+		x_do_cycles (2*cpucycleunit);
+//		fprintf ( stderr , "mem wait write after %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+	}
 
 	if (mode < 0)
 		put_long (addr, v);
@@ -241,7 +253,7 @@ void wait_cpu_cycle_write (uaecptr addr, int mode, uae_u32 v)
 	else if (mode == 0)
 		put_byte (addr, v);
 
-	x_do_cycles_post (CYCLE_UNIT, v);
+	x_do_cycles_post (2*CYCLE_UNIT, v);
 #endif						/* WINUAE_FOR_HATARI */
 }
 
@@ -443,11 +455,6 @@ void fixup_cpu (struct uae_prefs *p)
 	if (p->cpu_model >= 68040 && p->cachesize && p->cpu_compatible)
 		p->cpu_compatible = false;
 
-	if (p->cpu_model >= 68040 && p->cpu_cycle_exact) {
-		p->cpu_cycle_exact = 0;
-		error_log (_T("68040/060 cycle-exact is not supported."));
-	}
-
 	if ((p->cpu_model < 68030 || p->cachesize) && p->mmu_model) {
 		error_log (_T("MMU emulation requires 68030/040/060 and it is not JIT compatible."));
 		p->mmu_model = 0;
