@@ -9,6 +9,12 @@
   * Copyright 1999 Sam Jordan
   */
 
+#ifndef UAE_FPP_IEEE_BE_H
+#define UAE_FPP_IEEE_BE_H
+
+#include "uae/types.h"
+
+
 STATIC_INLINE double to_single (uae_u32 value)
 {
     union {
@@ -62,3 +68,5 @@ STATIC_INLINE void from_double(double src, uae_u32 * wrd1, uae_u32 * wrd2)
 
 /* Get the rest of the conversion functions defined.  */
 #include "fpp-unknown.h"
+
+#endif /* UAE_FPP_IEEE_BE_H */
