@@ -13,13 +13,12 @@
 const char Memory_fileid[] = "Hatari memory.c : " __DATE__ " " __TIME__;
 
 #include <SDL.h>
-#include "config.h"
+#include "main.h"
 #include "sysdeps.h"
 #include "hatari-glue.h"
 #include "maccess.h"
 #include "memory.h"
 
-#include "main.h"
 #include "tos.h"
 #include "ide.h"
 #include "ioMem.h"
@@ -57,8 +56,8 @@ static uae_u32 TTmem_mask;
 #define IOmem_mask  (IOmem_size - 1)
 
 /* Some prototypes: */
-static int STmem_check (uaecptr addr, uae_u32 size) REGPARAM;
-static uae_u8 *STmem_xlate (uaecptr addr) REGPARAM;
+static int REGPARAM3 STmem_check (uaecptr addr, uae_u32 size) REGPARAM;
+static uae_u8 REGPARAM3 *STmem_xlate (uaecptr addr) REGPARAM;
 
 
 
@@ -160,8 +159,6 @@ int addr_valid (const TCHAR *txt, uaecptr addr, uae_u32 len)
 	return 1;
 }
 
-static int illegal_count;
-
 static uae_u32 REGPARAM3 dummy_lget (uaecptr) REGPARAM;
 static uae_u32 REGPARAM3 dummy_wget (uaecptr) REGPARAM;
 static uae_u32 REGPARAM3 dummy_bget (uaecptr) REGPARAM;
@@ -244,9 +241,9 @@ void dummy_put (uaecptr addr, int size, uae_u32 val)
 #endif
 }
 
-uae_u32 dummy_get (uaecptr addr, int size, bool inst)
+uae_u32 dummy_get (uaecptr addr, int size, bool inst, uae_u32 defvalue)
 {
-	uae_u32 v = NONEXISTINGDATA;
+	uae_u32 v = defvalue;
 
 #ifndef WINUAE_FOR_HATARI
 #if FLASHEMU
@@ -307,7 +304,7 @@ static uae_u32 REGPARAM2 dummy_lget (uaecptr addr)
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 4, 0, 0);
-	return dummy_get (addr, 4, false);
+	return dummy_get (addr, 4, false, NONEXISTINGDATA);
 }
 uae_u32 REGPARAM2 dummy_lgeti (uaecptr addr)
 {
@@ -316,7 +313,7 @@ uae_u32 REGPARAM2 dummy_lgeti (uaecptr addr)
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 4, 0, 1);
-	return dummy_get (addr, 4, true);
+	return dummy_get (addr, 4, true, NONEXISTINGDATA);
 }
 
 static uae_u32 REGPARAM2 dummy_wget (uaecptr addr)
@@ -332,7 +329,7 @@ static uae_u32 REGPARAM2 dummy_wget (uaecptr addr)
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 2, 0, 0);
-	return dummy_get (addr, 2, false);
+	return dummy_get (addr, 2, false, NONEXISTINGDATA);
 }
 uae_u32 REGPARAM2 dummy_wgeti (uaecptr addr)
 {
@@ -341,7 +338,7 @@ uae_u32 REGPARAM2 dummy_wgeti (uaecptr addr)
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 2, 0, 1);
-	return dummy_get (addr, 2, true);
+	return dummy_get (addr, 2, true, NONEXISTINGDATA);
 }
 
 static uae_u32 REGPARAM2 dummy_bget (uaecptr addr)
@@ -351,7 +348,7 @@ static uae_u32 REGPARAM2 dummy_bget (uaecptr addr)
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 1, 0, 0);
-	return dummy_get (addr, 1, false);
+	return dummy_get (addr, 1, false, NONEXISTINGDATA);
 }
 
 static void REGPARAM2 dummy_lput (uaecptr addr, uae_u32 l)
@@ -390,7 +387,7 @@ static int REGPARAM2 dummy_check (uaecptr addr, uae_u32 size)
 	return 0;
 }
 
-static uae_u8 *dummy_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *dummy_xlate(uaecptr addr)
 {
     write_log("Your Atari program just did something terribly stupid:"
               " dummy_xlate($%x)\n", addr);
@@ -492,7 +489,7 @@ uae_u8 *REGPARAM3 sub_bank_xlate(uaecptr addr) REGPARAM
 
 /* **** This memory bank only generates bus errors **** */
 
-static uae_u32 BusErrMem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 BusErrMem_lget(uaecptr addr)
 {
     print_illegal_counted("Bus error lget", addr);
 
@@ -500,7 +497,7 @@ static uae_u32 BusErrMem_lget(uaecptr addr)
     return 0;
 }
 
-static uae_u32 BusErrMem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 BusErrMem_wget(uaecptr addr)
 {
     print_illegal_counted("Bus error wget", addr);
 
@@ -508,7 +505,7 @@ static uae_u32 BusErrMem_wget(uaecptr addr)
     return 0;
 }
 
-static uae_u32 BusErrMem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 BusErrMem_bget(uaecptr addr)
 {
     print_illegal_counted("Bus error bget", addr);
 
@@ -516,28 +513,28 @@ static uae_u32 BusErrMem_bget(uaecptr addr)
     return 0;
 }
 
-static void BusErrMem_lput(uaecptr addr, uae_u32 l)
+static void REGPARAM3 BusErrMem_lput(uaecptr addr, uae_u32 l)
 {
     print_illegal_counted("Bus error lput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
 }
 
-static void BusErrMem_wput(uaecptr addr, uae_u32 w)
+static void REGPARAM3 BusErrMem_wput(uaecptr addr, uae_u32 w)
 {
     print_illegal_counted("Bus error wput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
 }
 
-static void BusErrMem_bput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 BusErrMem_bput(uaecptr addr, uae_u32 b)
 {
     print_illegal_counted("Bus error bput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
 }
 
-static int BusErrMem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 BusErrMem_check(uaecptr addr, uae_u32 size)
 {
     if (illegal_mem)
 	write_log ("Bus error check at %08lx\n", (long)addr);
@@ -545,7 +542,7 @@ static int BusErrMem_check(uaecptr addr, uae_u32 size)
     return 0;
 }
 
-static uae_u8 *BusErrMem_xlate (uaecptr addr)
+static uae_u8 REGPARAM3 *BusErrMem_xlate (uaecptr addr)
 {
     write_log("Your Atari program just did something terribly stupid:"
               " BusErrMem_xlate($%x)\n", addr);
@@ -560,56 +557,56 @@ static uae_u8 *BusErrMem_xlate (uaecptr addr)
 /*static uae_u8 *STmemory;*/
 #define STmemory STRam
 
-static uae_u32 STmem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 STmem_lget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     return do_get_mem_long(STmemory + addr);
 }
 
-static uae_u32 STmem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 STmem_wget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     return do_get_mem_word(STmemory + addr);
 }
 
-static uae_u32 STmem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 STmem_bget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     return STmemory[addr];
 }
 
-static void STmem_lput(uaecptr addr, uae_u32 l)
+static void REGPARAM3 STmem_lput(uaecptr addr, uae_u32 l)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     do_put_mem_long(STmemory + addr, l);
 }
 
-static void STmem_wput(uaecptr addr, uae_u32 w)
+static void REGPARAM3 STmem_wput(uaecptr addr, uae_u32 w)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     do_put_mem_word(STmemory + addr, w);
 }
 
-static void STmem_bput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 STmem_bput(uaecptr addr, uae_u32 b)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     STmemory[addr] = b;
 }
 
-static int STmem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 STmem_check(uaecptr addr, uae_u32 size)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
     return (addr + size) <= STmem_size;
 }
 
-static uae_u8 *STmem_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *STmem_xlate(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
@@ -624,7 +621,7 @@ static uae_u8 *STmem_xlate(uaecptr addr)
  * 8 bytes of the ST memory are also a mirror of the TOS ROM, so they are write
  * protected!
  */
-static uae_u32 SysMem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 SysMem_lget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
@@ -638,12 +635,13 @@ static uae_u32 SysMem_lget(uaecptr addr)
     return do_get_mem_long(STmemory + addr);
 }
 
-static uae_u32 SysMem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 SysMem_wget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
-    if(addr < 0x800 && !regs.s)
+    /* Only CPU will trigger bus error if bit S=0, not the blitter */
+    if(addr < 0x800 && !regs.s && BusMode == BUS_MODE_CPU)
     {
       M68000_BusError(addr, 1, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
       return 0;
@@ -652,7 +650,7 @@ static uae_u32 SysMem_wget(uaecptr addr)
     return do_get_mem_word(STmemory + addr);
 }
 
-static uae_u32 SysMem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 SysMem_bget(uaecptr addr)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
@@ -666,7 +664,7 @@ static uae_u32 SysMem_bget(uaecptr addr)
     return STmemory[addr];
 }
 
-static void SysMem_lput(uaecptr addr, uae_u32 l)
+static void REGPARAM3 SysMem_lput(uaecptr addr, uae_u32 l)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
@@ -680,21 +678,28 @@ static void SysMem_lput(uaecptr addr, uae_u32 l)
     do_put_mem_long(STmemory + addr, l);
 }
 
-static void SysMem_wput(uaecptr addr, uae_u32 w)
+static void REGPARAM3 SysMem_wput(uaecptr addr, uae_u32 w)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
+    /* Only CPU will trigger bus error if bit S=0, not the blitter */
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
-      return;
+      if ( BusMode == BUS_MODE_CPU )
+      {
+	M68000_BusError(addr, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+	return;
+      }
+      /* If blitter writes < 0x8 then it should be ignored, else the write should be made */
+      else if ( ( BusMode == BUS_MODE_BLITTER ) && ( addr < 0x8 ) )
+	return;
     }
 
     do_put_mem_word(STmemory + addr, w);
 }
 
-static void SysMem_bput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 SysMem_bput(uaecptr addr, uae_u32 b)
 {
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
@@ -719,34 +724,34 @@ static void SysMem_bput(uaecptr addr, uae_u32 b)
  * prefetch register to decode the next opcode (tested on a real STF)
  */
 
-static uae_u32 VoidMem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 VoidMem_lget(uaecptr addr)
 {
     return 0;
 }
 
-static uae_u32 VoidMem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 VoidMem_wget(uaecptr addr)
 {
     return 0;
 }
 
-static uae_u32 VoidMem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 VoidMem_bget(uaecptr addr)
 {
     return 0;
 }
 
-static void VoidMem_lput(uaecptr addr, uae_u32 l)
+static void REGPARAM3 VoidMem_lput(uaecptr addr, uae_u32 l)
 {
 }
 
-static void VoidMem_wput(uaecptr addr, uae_u32 w)
+static void REGPARAM3 VoidMem_wput(uaecptr addr, uae_u32 w)
 {
 }
 
-static void VoidMem_bput (uaecptr addr, uae_u32 b)
+static void REGPARAM3 VoidMem_bput (uaecptr addr, uae_u32 b)
 {
 }
 
-static int VoidMem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 VoidMem_check(uaecptr addr, uae_u32 size)
 {
     if (illegal_mem)
 	write_log ("Void memory check at %08lx\n", (long)addr);
@@ -754,7 +759,7 @@ static int VoidMem_check(uaecptr addr, uae_u32 size)
     return 0;
 }
 
-static uae_u8 *VoidMem_xlate (uaecptr addr)
+static uae_u8 REGPARAM3 *VoidMem_xlate (uaecptr addr)
 {
     write_log("Your Atari program just did something terribly stupid:"
               " VoidMem_xlate($%x)\n", addr);
@@ -767,56 +772,56 @@ static uae_u8 *VoidMem_xlate (uaecptr addr)
 
 uae_u8 *TTmemory;
 
-static uae_u32 TTmem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 TTmem_lget(uaecptr addr)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     return do_get_mem_long(TTmemory + addr);
 }
 
-static uae_u32 TTmem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 TTmem_wget(uaecptr addr)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     return do_get_mem_word(TTmemory + addr);
 }
 
-static uae_u32 TTmem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 TTmem_bget(uaecptr addr)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     return TTmemory[addr];
 }
 
-static void TTmem_lput(uaecptr addr, uae_u32 l)
+static void REGPARAM3 TTmem_lput(uaecptr addr, uae_u32 l)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     do_put_mem_long(TTmemory + addr, l);
 }
 
-static void TTmem_wput(uaecptr addr, uae_u32 w)
+static void REGPARAM3 TTmem_wput(uaecptr addr, uae_u32 w)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     do_put_mem_word(TTmemory + addr, w);
 }
 
-static void TTmem_bput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 TTmem_bput(uaecptr addr, uae_u32 b)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     TTmemory[addr] = b;
 }
 
-static int TTmem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 TTmem_check(uaecptr addr, uae_u32 size)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
     return (addr + size) <= TTmem_size;
 }
 
-static uae_u8 *TTmem_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *TTmem_xlate(uaecptr addr)
 {
     addr -= TTmem_start & TTmem_mask;
     addr &= TTmem_mask;
@@ -828,56 +833,56 @@ static uae_u8 *TTmem_xlate(uaecptr addr)
 
 uae_u8 *ROMmemory;
 
-static uae_u32 ROMmem_lget(uaecptr addr)
+static uae_u32 REGPARAM3 ROMmem_lget(uaecptr addr)
 {
     addr -= ROMmem_start & ROMmem_mask;
     addr &= ROMmem_mask;
     return do_get_mem_long(ROMmemory + addr);
 }
 
-static uae_u32 ROMmem_wget(uaecptr addr)
+static uae_u32 REGPARAM3 ROMmem_wget(uaecptr addr)
 {
     addr -= ROMmem_start & ROMmem_mask;
     addr &= ROMmem_mask;
     return do_get_mem_word(ROMmemory + addr);
 }
 
-static uae_u32 ROMmem_bget(uaecptr addr)
+static uae_u32 REGPARAM3 ROMmem_bget(uaecptr addr)
 {
     addr -= ROMmem_start & ROMmem_mask;
     addr &= ROMmem_mask;
     return ROMmemory[addr];
 }
 
-static void ROMmem_lput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 ROMmem_lput(uaecptr addr, uae_u32 b)
 {
     print_illegal_counted("Illegal ROMmem lput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
 }
 
-static void ROMmem_wput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 ROMmem_wput(uaecptr addr, uae_u32 b)
 {
     print_illegal_counted("Illegal ROMmem wput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
 }
 
-static void ROMmem_bput(uaecptr addr, uae_u32 b)
+static void REGPARAM3 ROMmem_bput(uaecptr addr, uae_u32 b)
 {
     print_illegal_counted("Illegal ROMmem bput", addr);
 
     M68000_BusError(addr, 0, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
 }
 
-static int ROMmem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 ROMmem_check(uaecptr addr, uae_u32 size)
 {
     addr -= ROMmem_start & ROMmem_mask;
     addr &= ROMmem_mask;
     return (addr + size) <= ROMmem_size;
 }
 
-static uae_u8 *ROMmem_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *ROMmem_xlate(uaecptr addr)
 {
     addr -= ROMmem_start & ROMmem_mask;
     addr &= ROMmem_mask;
@@ -890,14 +895,14 @@ static uae_u8 *ROMmem_xlate(uaecptr addr)
 
 static uae_u8 *IdeMemory;
 
-static int IdeMem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 IdeMem_check(uaecptr addr, uae_u32 size)
 {
     addr -= IdeMem_start;
     addr &= IdeMem_mask;
     return (addr + size) <= IdeMem_size;
 }
 
-static uae_u8 *IdeMem_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *IdeMem_xlate(uaecptr addr)
 {
     addr -= IdeMem_start;
     addr &= IdeMem_mask;
@@ -910,14 +915,14 @@ static uae_u8 *IdeMem_xlate(uaecptr addr)
 
 uae_u8 *IOmemory;
 
-static int IOmem_check(uaecptr addr, uae_u32 size)
+static int REGPARAM3 IOmem_check(uaecptr addr, uae_u32 size)
 {
     addr -= IOmem_start;
     addr &= IOmem_mask;
     return (addr + size) <= IOmem_size;
 }
 
-static uae_u8 *IOmem_xlate(uaecptr addr)
+static uae_u8 REGPARAM3 *IOmem_xlate(uaecptr addr)
 {
     addr -= IOmem_start;
     addr &= IOmem_mask;
@@ -1239,7 +1244,7 @@ bool memory_region_bus_error ( uaecptr addr )
  *
  * Possible values for ce_banktype :
  *  CE_MEMBANK_CHIP16	shared between CPU and DMA, bus width = 16 bits
- *  CE_MEMBANK_CHIP32	shared between CPU and DMA, bus width = 16 bits (AGA chipset)
+ *  CE_MEMBANK_CHIP32	shared between CPU and DMA, bus width = 32 bits (AGA chipset)
  *  CE_MEMBANK_FAST16	accessible only to the CPU,  bus width = 16 bits
  *  CE_MEMBANK_FAST32 	accessible only to the CPU,  bus width = 32 bits
  *  CE_MEMBANK_CIA	Amiga only, for CIA chips
@@ -1281,7 +1286,6 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
 {
     int 	addr;
 
-    currprefs.address_space_24 = ConfigureParams.System.bAddressSpace24;	/* temp, do it in m68000.c */
     last_address_space_24 = currprefs.address_space_24;
 
     STmem_size = (nNewSTMemSize + 65535) & 0xFFFF0000;
@@ -1356,11 +1360,10 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     /* Handle extra RAM on TT and Falcon starting at 0x1000000 and up to 0x80000000 */
     /* This requires the CPU to use 32 bit addressing */
     TTmemory = NULL;
-    if ( ConfigureParams.System.bAddressSpace24 == false )
+    if (!currprefs.address_space_24)
     {
-	/* If there's no extra RAM on a TT, region 0x01000000 - 0x80000000 (2047 MB) must return bus errors */
-	if ( ConfigureParams.System.nMachineType == MACHINE_TT )
-	    map_banks_ce ( &BusErrMem_bank, TTmem_start >> 16, ( TTmem_end - TTmem_start ) >> 16, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
+	/* If there's no Fast-RAM, region 0x01000000 - 0x80000000 (2047 MB) must return bus errors */
+	map_banks_ce ( &BusErrMem_bank, TTmem_start >> 16, ( TTmem_end - TTmem_start ) >> 16, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
 
 	if ( TTmem_size > 0 )
 	{
@@ -1388,12 +1391,12 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     /* Depending on which ROM version we are using, the other ROM region is illegal! */
     if(nNewRomMemStart == 0xFC0000)
     {
-        map_banks_ce(&ROMmem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_CACHABLE);	/* [NP] FIXME test needed on real STF, could be FAST16 in fact */
+        map_banks_ce(&ROMmem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);	/* [NP] tested on real STF, no bus wait from ROM */
         map_banks_ce(&BusErrMem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
     }
     else if(nNewRomMemStart == 0xE00000)
     {
-        map_banks_ce(&ROMmem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_CACHABLE);	/* [NP] FIXME test needed on real STF, could be FAST16 in fact */
+        map_banks_ce(&ROMmem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);	/* [NP] tested on real STF, no bus wait from ROM */
         map_banks_ce(&BusErrMem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
     }
     else
@@ -1402,14 +1405,14 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     }
 
     /* Cartridge memory: */
-    map_banks_ce(&ROMmem_bank, 0xFA0000 >> 16, 0x2, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_CACHABLE);		/* [NP] FIXME test needed on real STF, could be FAST16 in fact */
+    map_banks_ce(&ROMmem_bank, 0xFA0000 >> 16, 0x2, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);		/* [NP] tested on real STF, no bus wait from cartridge */
     ROMmem_bank.baseaddr = ROMmemory;
     ROMmem_bank.mask = ROMmem_mask;
     ROMmem_bank.start = ROMmem_start;
 
     /* IO memory: */
-    map_banks_ce(&IOmem_bank, IOmem_start>>16, 0x1, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
-    IOmem_bank.baseaddr = IOmemory;
+    map_banks_ce(&IOmem_bank, IOmem_start>>16, 0x1, 0, CE_MEMBANK_FAST16, CE_MEMBANK_NOT_CACHABLE);	/* [NP] tested on real STF, no bus wait for IO memory */
+    IOmem_bank.baseaddr = IOmemory;									/* except for some shifter registers */
     IOmem_bank.mask = IOmem_mask;
     IOmem_bank.start = IOmem_start;
 
@@ -1422,14 +1425,12 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     /* Illegal memory regions cause a bus error on the ST: */
     map_banks_ce(&BusErrMem_bank, 0xF10000 >> 16, 0x9, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
 
-
-    /* If MMU is disabled on TT/Falcon and we use full 32 bit addressing */
-    /* then we remap memory 00xxxxxx to FFxxxxxx (as a minimal replacement */
-    /* for the MMU's tables). Else, we get some crashes when booting TOS 3 and 4 */
-    if ( ( ConfigureParams.System.bAddressSpace24 == false )
-      && ( ConfigureParams.System.bMMU == false )
-      && ( ( ConfigureParams.System.nMachineType == MACHINE_TT )
-	|| ( ConfigureParams.System.nMachineType == MACHINE_FALCON ) ) )
+    /* According to the "Atari TT030 Hardware Reference Manual", the
+     * lowest 16 MBs (i.e. the 24-bit address space) are always mirrored
+     * to 0xff000000, so we remap memory 00xxxxxx to FFxxxxxx here. If not,
+     * we'd get some crashes when booting TOS 3 and 4 (e.g. both TOS 3.06
+     * and TOS 4.04 touch 0xffff8606 before setting up the MMU tables) */
+    if (!currprefs.address_space_24)
     {
       /* Copy all 256 banks 0x0000-0x00FF to banks 0xFF00-0xFFFF */
       for ( addr=0x0 ; addr<=0x00ffffff ; addr+=0x10000 )
@@ -1545,6 +1546,11 @@ static void map_banks2 (addrbank *bank, int start, int size, int realsize, int q
 #endif
 			}
 			put_mem_bank ((bnr + hioffs) << 16, bank, realstart << 16);
+
+	  		/* Copy the CE parameters for bank/start */
+			ce_banktype[ (bnr + hioffs) ] = ce_banktype[ start ];
+			ce_cachable[ (bnr + hioffs) ] = ce_cachable[ start ];
+//printf ( "ce copy %x %x\n" , ce_banktype[ (bnr + hioffs) ] , ce_cachable[ (bnr + hioffs) ] );
 			real_left--;
 		}
 	}
@@ -1570,8 +1576,8 @@ void map_banks_nojitdirect (addrbank *bank, int start, int size, int realsize)
 
 void map_banks_ce (addrbank *bank, int start, int size, int realsize , int banktype, int cachable )
 {
-	map_banks2 (bank, start, size, realsize, 0);
 	fill_ce_banks (start, size, banktype, cachable );
+	map_banks2 (bank, start, size, realsize, 0);
 }
 
 
