@@ -6,11 +6,12 @@
 * Copyright 1995 Bernd Schmidt
 */
 
-#ifndef NEWCPU_H
-#define NEWCPU_H
+#ifndef UAE_NEWCPU_H
+#define UAE_NEWCPU_H
 
+#include "uae/types.h"
 #include "readcpu.h"
-#include "m68k.h"
+#include "machdep/m68k.h"
 #include "compat.h"
 #include "maccess.h"
 #include "events.h"
@@ -40,7 +41,6 @@
 #define SPCFLAG_MODE_CHANGE 0x800
 #define SPCFLAG_DSP 0x1000
 
-
 #ifdef WITH_SOFTFLOAT
 #include <softfloat.h>
 #endif
@@ -96,8 +96,16 @@ struct cputbl {
 };
 
 #ifdef JIT
+#define MAX_JIT_CACHE 16384
 typedef uae_u32 REGPARAM3 compop_func (uae_u32) REGPARAM;
 
+#define COMP_OPCODE_ISJUMP      0x0001
+#define COMP_OPCODE_LONG_OPCODE 0x0002
+#define COMP_OPCODE_CMOV        0x0004
+#define COMP_OPCODE_ISADDX      0x0008
+#define COMP_OPCODE_ISCJUMP     0x0010
+#define COMP_OPCODE_USES_FPU    0x0020
+
 struct comptbl {
 	compop_func *handler;
 	uae_u32 opcode;
@@ -181,7 +189,7 @@ struct regstruct
 	uae_u32 instruction_pc;
 
 	uae_u16 irc, ir, db;
-	uae_u32 spcflags;
+	volatile uae_atomic spcflags;
 	uae_u32 last_prefetch;
 	uae_u32 chipset_latch_rw;
 	uae_u32 chipset_latch_read;
@@ -290,6 +298,23 @@ extern int mmu_enabled, mmu_triggered;
 extern int cpu_cycles;
 extern int cpucycleunit;
 extern int m68k_pc_indirect;
+
+#ifndef WINUAE_FOR_HATARI
+STATIC_INLINE void set_special_exter(uae_u32 x)
+{
+	atomic_or(&regs.spcflags, x);
+}
+STATIC_INLINE void set_special (uae_u32 x)
+{
+	atomic_or(&regs.spcflags, x);
+	cycles_do_special ();
+}
+
+STATIC_INLINE void unset_special (uae_u32 x)
+{
+	atomic_and(&regs.spcflags, ~x);
+}
+#else
 STATIC_INLINE void set_special (uae_u32 x)
 {
 	regs.spcflags |= x;
@@ -301,6 +326,8 @@ STATIC_INLINE void unset_special (uae_u32 x)
 	regs.spcflags &= ~x;
 }
 
+#endif
+
 #define m68k_dreg(r,num) ((r).regs[(num)])
 #define m68k_areg(r,num) (((r).regs + 8)[(num)])
 
@@ -568,7 +595,6 @@ extern void set_cpu_caches (bool flush);
 extern void flush_cpu_caches(bool flush);
 extern void flush_cpu_caches_040(uae_u16 opcode);
 extern void REGPARAM3 MakeSR (void) REGPARAM;
-extern void SetSR (uae_u16 sr);
 extern void REGPARAM3 MakeFromSR (void) REGPARAM;
 extern void REGPARAM3 Exception (int) REGPARAM;
 extern void REGPARAM3 ExceptionL (int, uaecptr) REGPARAM;
@@ -614,17 +640,18 @@ extern void fpux_restore (int*);
 extern bool fpu_get_constant(fpdata *fp, int cr);
 extern int fpp_cond(int condition);
 
-extern void exception3_read (uae_u32 opcode, uaecptr addr);
-extern void exception3_write (uae_u32 opcode, uaecptr addr);
+extern void exception3_read(uae_u32 opcode, uaecptr addr);
+extern void exception3_write(uae_u32 opcode, uaecptr addr);
 extern void exception3_notinstruction(uae_u32 opcode, uaecptr addr);
 extern void exception3i (uae_u32 opcode, uaecptr addr);
 extern void exception3b (uae_u32 opcode, uaecptr addr, bool w, bool i, uaecptr pc);
 extern void exception2 (uaecptr addr, bool read, int size, uae_u32 fc);
-extern void exception2_fake (uaecptr addr);
 extern void m68k_reset (void);
 extern void cpureset (void);
 extern void cpu_halt (int id);
-extern void cpu_sleep_millis(int ms);
+extern int cpu_sleep_millis(int ms);
+extern void cpu_change(int newmodel);
+extern void cpu_fallback(int mode);
 
 extern void fill_prefetch (void);
 extern void fill_prefetch_020 (void);
@@ -639,32 +666,38 @@ extern void fill_prefetch_030 (void);
 /* 68060 */
 extern const struct cputbl op_smalltbl_0_ff[];
 extern const struct cputbl op_smalltbl_40_ff[];
+extern const struct cputbl op_smalltbl_50_ff[];
 extern const struct cputbl op_smalltbl_24_ff[]; // CE
 extern const struct cputbl op_smalltbl_33_ff[]; // MMU
 /* 68040 */
 extern const struct cputbl op_smalltbl_1_ff[];
 extern const struct cputbl op_smalltbl_41_ff[];
+extern const struct cputbl op_smalltbl_51_ff[];
 extern const struct cputbl op_smalltbl_25_ff[]; // CE
 extern const struct cputbl op_smalltbl_31_ff[]; // MMU
 /* 68030 */
 extern const struct cputbl op_smalltbl_2_ff[];
 extern const struct cputbl op_smalltbl_42_ff[];
+extern const struct cputbl op_smalltbl_52_ff[];
 extern const struct cputbl op_smalltbl_22_ff[]; // prefetch
 extern const struct cputbl op_smalltbl_23_ff[]; // CE
 extern const struct cputbl op_smalltbl_32_ff[]; // MMU
 /* 68020 */
 extern const struct cputbl op_smalltbl_3_ff[];
 extern const struct cputbl op_smalltbl_43_ff[];
+extern const struct cputbl op_smalltbl_53_ff[];
 extern const struct cputbl op_smalltbl_20_ff[]; // prefetch
 extern const struct cputbl op_smalltbl_21_ff[]; // CE
 /* 68010 */
 extern const struct cputbl op_smalltbl_4_ff[];
 extern const struct cputbl op_smalltbl_44_ff[];
+extern const struct cputbl op_smalltbl_54_ff[];
 extern const struct cputbl op_smalltbl_11_ff[]; // prefetch
 extern const struct cputbl op_smalltbl_13_ff[]; // CE
 /* 68000 */
 extern const struct cputbl op_smalltbl_5_ff[];
 extern const struct cputbl op_smalltbl_45_ff[];
+extern const struct cputbl op_smalltbl_55_ff[];
 extern const struct cputbl op_smalltbl_12_ff[]; // prefetch
 extern const struct cputbl op_smalltbl_14_ff[]; // CE
 
@@ -674,11 +707,11 @@ extern cpuop_func *cpufunctbl[65536] ASM_SYM_FOR_FUNC ("cpufunctbl");
 extern void flush_icache(uaecptr, int);
 extern void flush_icache_hard(uaecptr, int);
 extern void compemu_reset(void);
-extern bool check_prefs_changed_comp (void);
 #else
 #define flush_icache(uaecptr, int) do {} while (0)
 #define flush_icache_hard(uaecptr, int) do {} while (0)
 #endif
+bool check_prefs_changed_comp (bool);
 extern void flush_dcache (uaecptr, int);
 #ifdef WINUAE_FOR_HATARI
 extern void flush_instr_cache (uaecptr, int);
@@ -698,13 +731,30 @@ extern bool is_cpu_tracer (void);
 extern bool set_cpu_tracer (bool force);
 extern bool can_cpu_tracer (void);
 
+#define CPU_HALT_PPC_ONLY -1
+#define CPU_HALT_BUS_ERROR_DOUBLE_FAULT 1
+#define CPU_HALT_DOUBLE_FAULT 2
+#define CPU_HALT_OPCODE_FETCH_FROM_NON_EXISTING_ADDRESS 3
+#define CPU_HALT_ACCELERATOR_CPU_FALLBACK 4
+#define CPU_HALT_ALL_CPUS_STOPPED 5
+#define CPU_HALT_FAKE_DMA 6
+#define CPU_HALT_AUTOCONFIG_CONFLICT 7
+#define CPU_HALT_PCI_CONFLICT 8
+#define CPU_HALT_CPU_STUCK 9
+#define CPU_HALT_SSP_IN_NON_EXISTING_ADDRESS 10
+
+void cpu_semaphore_get(void);
+void cpu_semaphore_release(void);
+bool execute_other_cpu(int until);
+void execute_other_cpu_single(void);
+
 #ifdef WINUAE_FOR_HATARI
 /*** Hatari ***/
 
 /* Family of the latest instruction executed (to check for pairing) */
 extern int OpcodeFamily;			/* see instrmnem in readcpu.h */
 
-/* How many cycles to add to the current instruction in case a "misaligned" bus acces is made */
+/* How many cycles to add to the current instruction in case a "misaligned" bus access is made */
 /* (e.g. used when addressing mode is d8(an,ix)) */
 extern int BusCyclePenalty;
 
@@ -714,5 +764,4 @@ extern FILE *console_out_FILE;
 /*** Hatari ***/
 #endif
 
-#endif
-
+#endif /* UAE_NEWCPU_H */
