@@ -25,13 +25,14 @@
 #define	EXCEPT_CHK	 (1<<4)
 #define	EXCEPT_TRAPV	 (1<<5)
 #define	EXCEPT_PRIVILEGE (1<<6)
-#define	EXCEPT_NOHANDLER (1<<7)
+#define	EXCEPT_TRACE     (1<<7)
+#define	EXCEPT_NOHANDLER (1<<8)
 
 /* DSP exception flags */
-#define EXCEPT_DSP	 (1<<8)
+#define EXCEPT_DSP	 (1<<9)
 
 /* whether to enable exception debugging on autostart */
-#define EXCEPT_AUTOSTART (1<<9)
+#define EXCEPT_AUTOSTART (1<<10)
 
 /* general flags */
 #define	EXCEPT_NONE	 (0)
@@ -172,6 +173,8 @@ extern char *Log_MatchTrace(const char *text, int state);
 
 #define TRACE_OS_BASE		 (1ll<<51)
 
+#define TRACE_SCSIDRV		 (1ll<<52)
+    
 #define	TRACE_NONE		 (0)
 #define	TRACE_ALL		 (~0)
 
@@ -200,17 +203,15 @@ extern Uint64 LogTraceFlags;
 
 #if ENABLE_TRACING
 
-#ifndef _VCWIN_
-#define	LOG_TRACE(level, args...) \
-	if (unlikely(LogTraceFlags & (level))) { fprintf(TraceFile, args); fflush(TraceFile); }
-#endif
+#define	LOG_TRACE(level, ...) \
+	if (unlikely(LogTraceFlags & (level))) { fprintf(TraceFile, __VA_ARGS__); fflush(TraceFile); }
+
 #define LOG_TRACE_LEVEL( level )	(unlikely(LogTraceFlags & (level)))
 
 #else		/* ENABLE_TRACING */
 
-#ifndef _VCWIN_
-#define LOG_TRACE(level, args...)	{}
-#endif
+#define LOG_TRACE(level, ...)	{}
+
 #define LOG_TRACE_LEVEL( level )	(0)
 
 #endif		/* ENABLE_TRACING */
@@ -219,9 +220,7 @@ extern Uint64 LogTraceFlags;
  * In code it's used in such a way that it will be optimized away when tracing
  * is disabled.
  */
-#ifndef _VCWIN_
-#define LOG_TRACE_PRINT(args...)	fprintf(TraceFile , args)
-#endif
+#define LOG_TRACE_PRINT(...)	fprintf(TraceFile , __VA_ARGS__)
 
 
 #endif		/* HATARI_LOG_H */
