@@ -1000,8 +1000,7 @@ static void	STX_FreeSaveSectorsStruct ( STX_SAVE_SECTOR_STRUCT *pSaveSectorsStru
 	if ( pSaveSectorsStruct[ Nb ].StructIsUsed == 0 )
 		return;						/* This structure is already free */
 
-	if ( pSaveSectorsStruct[ Nb ].pData )
-		free ( pSaveSectorsStruct[ Nb ].pData );
+	free(pSaveSectorsStruct[Nb].pData);
 
 	pSaveSectorsStruct[ Nb ].StructIsUsed = 0;
 }
@@ -1033,10 +1032,8 @@ static void	STX_FreeSaveTracksStructAll ( STX_SAVE_TRACK_STRUCT *pSaveTracksStru
  */
 static void	STX_FreeSaveTracksStruct ( STX_SAVE_TRACK_STRUCT *pSaveTracksStruct , int Nb )
 {
-	if ( pSaveTracksStruct[ Nb ].pDataWrite )
-		free ( pSaveTracksStruct[ Nb ].pDataWrite );
-	if ( pSaveTracksStruct[ Nb ].pDataRead )
-		free ( pSaveTracksStruct[ Nb ].pDataRead );
+	free(pSaveTracksStruct[Nb].pDataWrite);
+	free(pSaveTracksStruct[Nb].pDataRead);
 }
 
 
@@ -1496,7 +1493,7 @@ Uint32	FDC_GetCyclesPerRev_FdcCycles_STX ( Uint8 Drive , Uint8 Track , Uint8 Sid
  * ID Field of the track (which simulates a full spin of the floppy).
  * We also store the next sector's number into NextSectorStruct_Nbr,
  * the next sector's number into NextSector_ID_Field_SR, the next track's number
- * into NextSector_ID_Field_TR, the next sector's lenght into
+ * into NextSector_ID_Field_TR, the next sector's length into
  * NextSector_ID_Field_LEN and if the CRC is correct or not into NextSector_ID_Field_CRC_OK.
  * This function assumes the sectors of each track are sorted in ascending order
  * using BitPosition.
