@@ -30,7 +30,11 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 #include <sys/statvfs.h>
 #endif
 #include <sys/types.h>
+#if HAVE_UTIME_H
 #include <utime.h>
+#elif HAVE_SYS_UTIME_H
+#include <sys/utime.h>
+#endif
 #include <time.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -50,7 +54,6 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "printer.h"
-#include "rs232.h"
 #include "statusbar.h"
 #include "scandir.h"
 #include "stMemory.h"
@@ -350,12 +353,11 @@ static bool fsfirst_match(const char *pat, const char *name)
 
 	if (name[0] == '.')
 		return false;           /* skip .* files */
-	if (strcmp(pat,"*.*")==0)
-		return true;            /* match everything */
-	if (strcasecmp(pat,name)==0)
-		return true;            /* exact case insensitive match */
 
-	dot = strrchr(name, '.');	/* '*' matches everything except _last_ '.' */
+	dot = strrchr(name, '.');	/* '*' matches everything except last dot in name */
+	if (dot && p[0] == '*' && p[1] == 0)
+		return false;		/* plain '*' must not match anything with extension */
+
 	while (*n)
 	{
 		if (*p=='*')
@@ -364,23 +366,27 @@ static bool fsfirst_match(const char *pat, const char *name)
 				n++;
 			p++;
 		}
-		else
+		else if (*p=='?' && *n)
 		{
-			if (*p=='?' && *n)
-			{
-				n++;
-				p++;
-			}
-			else
-			{
-				if (toupper((unsigned char)*p++) != toupper((unsigned char)*n++))
-					return false;
-			}
+			n++;
+			p++;
 		}
+		else if (toupper((unsigned char)*p++) != toupper((unsigned char)*n++))
+			return false;
 	}
 
-	/* The name matches the pattern if it ends here, too */
-	return (*p == 0 || (*p == '*' && *(p+1) == 0));
+	/* printf("'%s': '%s' -> '%s' : '%s' -> %d\n", name, pat, n, p); */
+
+	/* The traversed name matches the pattern, if pattern also
+	 * ends here, or with '*'. '*' for extension matches also
+	 * filenames without extension, so pattern ending with
+	 * '.*' will also be a match.
+	 */
+	return (
+		(p[0] == 0) ||
+		(p[0] == '*' && p[1] == 0) ||
+		(p[0] == '.' && p[1] == '*' && p[2] == 0)
+	       );
 }
 
 
@@ -475,12 +481,12 @@ void GemDOS_Init(void)
 
 	/* Clear handles structure */
 	memset(FileHandles, 0, sizeof(FileHandles));
-	for(i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+	for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 	{
 		GemDOS_UnforceFileHandle(i);
 	}
 	/* Clear DTAs */
-	for(i = 0; i < ARRAYSIZE(InternalDTAs); i++)
+	for(i = 0; i < ARRAY_SIZE(InternalDTAs); i++)
 	{
 		InternalDTAs[i].bUsed = false;
 		InternalDTAs[i].nentries = 0;
@@ -499,11 +505,11 @@ void GemDOS_Reset(void)
 	int i;
 
 	/* Init file handles table */
-	for (i = 0; i < ARRAYSIZE(FileHandles); i++)
+	for (i = 0; i < ARRAY_SIZE(FileHandles); i++)
 	{
 		GemDOS_CloseFileHandle(i);
 	}
-	for(i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+	for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 	{
 		GemDOS_UnforceFileHandle(i);
 	}
@@ -513,6 +519,19 @@ void GemDOS_Reset(void)
 	}
 	DTAIndex = 0;
 
+	if (emudrives)
+	{
+		for (i = 0; i < MAX_HARDDRIVES; i++)
+		{
+			if (emudrives[i])
+			{
+				/* Initialize current directory to the root of the drive */
+				strcpy(emudrives[i]->fs_currpath, emudrives[i]->hd_emulation_dir);
+				File_AddSlashToEndFileName(emudrives[i]->fs_currpath);
+			}
+		}
+	}
+
 	/* Reset */
 	bInitGemDOS = false;
 	CurrentDrive = nBootDrive;
@@ -529,6 +548,8 @@ static bool GEMDOS_DoesHostDriveFolderExist(char* lpstrPath, int iDrive)
 {
 	bool bExist = false;
 
+	Log_Printf(LOG_DEBUG, "Checking GEMDOS %c: HDD: %s\n", 'A'+iDrive, lpstrPath);
+
 	if (access(lpstrPath, F_OK) != 0 )
 	{
 		/* Try lower case drive letter instead */
@@ -545,6 +566,10 @@ static bool GEMDOS_DoesHostDriveFolderExist(char* lpstrPath, int iDrive)
 		{
 			bExist = true;
 		}
+		else
+		{
+			Log_Printf(LOG_WARN, "Not suitable as GEMDOS HDD dir: %s\n", lpstrPath);
+		}
 	}
 
 	return bExist;
@@ -567,7 +592,8 @@ static bool GemDOS_DetermineMaxPartitions(int *pnMaxDrives)
 
 	/* Scan through the main directory to see whether there are just single
 	 * letter sub-folders there (then use multi-partition mode) or if
-	 * arbitrary sub-folders are there (then use single-partition mode */
+	 * arbitrary sub-folders are there (then use single-partition mode)
+	 */
 	count = scandir(ConfigureParams.HardDisk.szHardDiskDirectories[0], &files, 0, alphasort);
 	if (count < 0)
 	{
@@ -644,13 +670,12 @@ void GemDOS_InitDrives(void)
 	/* intialize data for harddrive emulation: */
 	if (nMaxDrives > 0 && !emudrives)
 	{
-		emudrives = malloc(MAX_HARDDRIVES * sizeof(EMULATEDDRIVE *));
+		emudrives = calloc(MAX_HARDDRIVES, sizeof(EMULATEDDRIVE *));
 		if (!emudrives)
 		{
 			perror("GemDOS_InitDrives");
 			return;
 		}
-		memset(emudrives, 0, MAX_HARDDRIVES * sizeof(EMULATEDDRIVE *));
 	}
 
 	ImagePartitions = nAcsiPartitions + nIDEPartitions;
@@ -695,10 +720,6 @@ void GemDOS_InitDrives(void)
 		// drive letter/number exists...
 		if (GEMDOS_DoesHostDriveFolderExist(emudrives[i]->hd_emulation_dir, DriveNumber))
 		{
-			/* initialize current directory string, too (initially the same as hd_emulation_dir) */
-			strcpy(emudrives[i]->fs_currpath, emudrives[i]->hd_emulation_dir);
-			File_AddSlashToEndFileName(emudrives[i]->fs_currpath);    /* Needs trailing slash! */
-
 			/* map drive */
 			Log_Printf(LOG_INFO, "GEMDOS HDD emulation, %c: <-> %s.\n",
 				   'A'+DriveNumber, emudrives[i]->hd_emulation_dir);
@@ -826,11 +847,11 @@ void GemDOS_MemorySnapShot_Capture(bool bSave)
 	if (!bSave)
 	{
 		/* Clear file handles  */
-		for(i = 0; i < ARRAYSIZE(FileHandles); i++)
+		for(i = 0; i < ARRAY_SIZE(FileHandles); i++)
 		{
 			GemDOS_CloseFileHandle(i);
 		}
-		for(i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+		for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 		{
 			GemDOS_UnforceFileHandle(i);
 		}
@@ -847,7 +868,7 @@ static int GemDOS_FindFreeFileHandle(void)
 	int i;
 
 	/* Scan our file list for free slot */
-	for(i = 0; i < ARRAYSIZE(FileHandles); i++)
+	for(i = 0; i < ARRAY_SIZE(FileHandles); i++)
 	{
 		if (!FileHandles[i].bUsed)
 			return i;
@@ -884,7 +905,7 @@ static int GemDOS_GetValidFileHandle(int Handle)
 	int Forced = -1;
 
 	/* Has handle been aliased with Fforce()? */
-	if (Handle >= 0 && Handle < ARRAYSIZE(ForcedHandles)
+	if (Handle >= 0 && Handle < ARRAY_SIZE(ForcedHandles)
 	    && ForcedHandles[Handle].Handle != UNFORCED_HANDLE)
 	{
 		if (GemDOS_BasepageMatches(ForcedHandles[Handle].Basepage))
@@ -905,7 +926,7 @@ static int GemDOS_GetValidFileHandle(int Handle)
 		Handle -= BASE_FILEHANDLE;
 	}
 	/* handle is valid for current program and in our handle table? */
-	if (Handle >= 0 && Handle < ARRAYSIZE(FileHandles)
+	if (Handle >= 0 && Handle < ARRAY_SIZE(FileHandles)
 	    && FileHandles[Handle].bUsed)
 	{
 		Uint32 current = STMemory_ReadLong(act_pd);
@@ -1087,10 +1108,11 @@ static int clip_to_83(char *name)
  */
 static bool add_path_component(char *path, int maxlen, const char *origname, bool is_dir)
 {
-	char *tmp, *match, name[strlen(origname) + 3];
+	char *tmp, *match;
 	int dot, namelen, pathlen;
 	int (*chr_conv)(int);
 	bool modified;
+	char *name = alloca(strlen(origname) + 3);
 
 	/* append separator */
 	pathlen = strlen(path);
@@ -1342,7 +1364,7 @@ void GemDOS_CreateHardDriveFileName(int Drive, const char *pszFileName,
 		if ((s = strchr(filename, '\\')))
 		{
 			int dirlen = s - filename;
-			char dirname[dirlen+1];
+			char *dirname = alloca(dirlen + 1);
 			/* copy dirname */
 			strncpy(dirname, filename, dirlen);
 			dirname[dirlen] = '\0';
@@ -1554,7 +1576,7 @@ static Uint32 errno2gemdos(const int error, const etype_t etype)
 	case EPERM:
 	case EACCES:
 	case EROFS:
-		return GEMDOS_EACCDN;        /* Acess denied */
+		return GEMDOS_EACCDN;        /* Access denied */
 	default:
 		return GEMDOS_ERROR;         /* Misc error */
 	}
@@ -2028,7 +2050,7 @@ static bool GemDOS_Close(Uint32 Params)
 	GemDOS_CloseFileHandle(Handle);
 
 	/* unalias handle */
-	for (i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+	for (i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 	{
 		if (ForcedHandles[i].Handle == Handle)
 			GemDOS_UnforceFileHandle(i);
@@ -2456,9 +2478,9 @@ static bool GemDOS_Force(Uint32 Params)
 		/* assume it was TOS one -> let TOS handle it */
 		return false;
 	}
-	if (std < 0 || std >= ARRAYSIZE(ForcedHandles))
+	if (std < 0 || std >= ARRAY_SIZE(ForcedHandles))
 	{
-		Log_Printf(LOG_WARN, "Warning: forcing of non-standard %d (> %d) handle ignored.\n", std, ARRAYSIZE(ForcedHandles));
+		Log_Printf(LOG_WARN, "Warning: forcing of non-standard %d (> %d) handle ignored.\n", std, ARRAY_SIZE(ForcedHandles));
 		return false;
 	}
 	/* mark given standard handle redirected by this process */
@@ -2927,7 +2949,7 @@ static void GemDOS_TerminateClose(void)
 	Uint32 current = STMemory_ReadLong(act_pd);
 
 	closed = 0;
-	for (i = 0; i < ARRAYSIZE(FileHandles); i++)
+	for (i = 0; i < ARRAY_SIZE(FileHandles); i++)
 	{
 		if (FileHandles[i].Basepage == current)
 		{
@@ -2936,7 +2958,7 @@ static void GemDOS_TerminateClose(void)
 		}
 	}
 	unforced = 0;
-	for (i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+	for (i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 	{
 		if (ForcedHandles[i].Basepage == current)
 		{
@@ -3091,7 +3113,7 @@ static const char* GemDOS_Opcode2Name(Uint16 opcode)
 		"Frename",
 		"Fdatime"
 	};
-	if (opcode < ARRAYSIZE(names))
+	if (opcode < ARRAY_SIZE(names))
 		return names[opcode];
 	return "MiNT?";
 }
@@ -3139,7 +3161,7 @@ void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 	}
 
 	fputs("\nInternal Fsfirst() DTAs:\n", fp);
-	for(used = i = 0; i < ARRAYSIZE(InternalDTAs); i++)
+	for(used = i = 0; i < ARRAY_SIZE(InternalDTAs); i++)
 	{
 		int j, centry, entries;
 
@@ -3163,7 +3185,7 @@ void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 		fputs("- None in use.\n", fp);
 
 	fputs("\nOpen GEMDOS HDD file handles:\n", fp);
-	for (used = i = 0; i < ARRAYSIZE(FileHandles); i++)
+	for (used = i = 0; i < ARRAY_SIZE(FileHandles); i++)
 	{
 		if (!FileHandles[i].bUsed)
 			continue;
@@ -3174,7 +3196,7 @@ void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 	if (!used)
 		fputs("- None.\n", fp);
 	fputs("\nForced GEMDOS HDD file handles:\n", fp);
-	for (used = i = 0; i < ARRAYSIZE(ForcedHandles); i++)
+	for (used = i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
 	{
 		if (ForcedHandles[i].Handle == UNFORCED_HANDLE)
 			continue;
@@ -3433,6 +3455,9 @@ void GemDOS_OpCode(void)
  */
 void GemDOS_Boot(void)
 {
+	if (bInitGemDOS)
+		GemDOS_Reset();
+
 	bInitGemDOS = true;
 
 	LOG_TRACE(TRACE_OS_GEMDOS, "Gemdos_Boot() at PC 0x%X\n", M68000_GetPC() );
@@ -3454,7 +3479,8 @@ void GemDOS_Boot(void)
 	}
 	else
 	{
-		act_pd = STMemory_ReadLong(TosAddress + 0x28);
+		Uint32 osAddress = STMemory_ReadLong(0x4f2);
+		act_pd = STMemory_ReadLong(osAddress + 0x28);
 	}
 
 	/* Save old GEMDOS handler address */
