@@ -623,7 +623,9 @@ static void HDC_EmulateCommandPacket(SCSI_CTRLR *ctr)
  * - Support also Atari ICD (12 entries, at offset 0x156) and
  *   extended partition schemes.  Linux kernel has code for those:
  *   http://lxr.free-electrons.com/source/block/partitions/atari.c
- * - Support partition tables with other endianess
+ *   Normal and extended partition tables are described in:
+ *   http://dev-docs.atariforge.org/files/AHDI_3_RN_4-18-1990.pdf
+ * - Support partition tables with other endianness
  */
 int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 {
@@ -654,15 +656,15 @@ int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 		{
 			boot = pinfo[0];
 			ptype = pinfo[4];
-			start = HDC_ReadInt32(pinfo, 8);
-			sectors = HDC_ReadInt32(pinfo, 12);
+			start = SDL_SwapLE32(*(long*)(pinfo+8));
+			sectors = SDL_SwapLE32(*(long*)(pinfo+12));
 			total += sectors;
-			LOG_TRACE(tracelevel, "- Partition %d: type=0x%02x, start=0x%08x, size=%d MB %s\n",
-				  i, ptype, start, sectors/2048, boot ? "(boot)" : "");
+			LOG_TRACE(tracelevel, "- Partition %d: type=0x%02x, start=0x%08x, size=%.1f MB %s%s\n",
+				  i, ptype, start, sectors/2048.0, boot ? "(boot)" : "", sectors ? "" : "(invalid)");
 			if (ptype)
 				parts++;
 		}
-		LOG_TRACE(tracelevel, "- Total size: %i MB in %d partitions\n", total/2048, parts);
+		LOG_TRACE(tracelevel, "- Total size: %.1f MB in %d partitions\n", total/2048.0, parts);
 	}
 	else
 	{
@@ -674,6 +676,7 @@ int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 		 */
 		char c, pid[4];
 		int j, flags;
+		bool extended;
 
 		LOG_TRACE(tracelevel, "ATARI MBR:\n");
 		pinfo = bootsector + 0x1C6;
@@ -688,15 +691,19 @@ int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 				pid[j] = c;
 			}
 			pid[3] = '\0';
+			extended = strcmp("XGM", pid) == 0;
 			start = HDC_ReadInt32(pinfo, 4);
 			sectors = HDC_ReadInt32(pinfo, 8);
-			LOG_TRACE(tracelevel, "- Partition %d: ID=%s, start=0x%08x, size=%d MB, flags=0x%x\n",
-				  i, pid, start, sectors/2048, flags);
+			LOG_TRACE(tracelevel,
+				  "- Partition %d: ID=%s, start=0x%08x, size=%.1f MB, flags=0x%x %s%s\n",
+				  i, pid, start, sectors/2048.0, flags,
+				  (flags & 0x80) ? "(boot)": "",
+				  extended ? "(extended)" : "");
 			if (flags & 0x1)
 				parts++;
 		}
 		total = HDC_ReadInt32(bootsector, 0x1C2);
-		LOG_TRACE(tracelevel, "- Total size: %i MB in %d partitions\n", total/2048, parts);
+		LOG_TRACE(tracelevel, "- Total size: %.1f MB in %d partitions\n", total/2048.0, parts);
 	}
 
 	if (fseeko(fp, offset, SEEK_SET) != 0)
@@ -704,6 +711,49 @@ int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 	return parts;
 }
 
+/**
+ * Check file size for sane values (non-zero, multiple of 512),
+ * and return the size
+ */
+off_t HDC_CheckAndGetSize(const char *filename)
+{
+	off_t filesize;
+	char shortname[48];
+
+	File_ShrinkName(shortname, filename, sizeof(shortname));
+
+	filesize = File_Length(filename);
+	if (filesize < 0)
+	{
+		Log_AlertDlg(LOG_ERROR, "Unable to get size of HD image file\n'%s'!",
+		             shortname);
+		if (sizeof(off_t) < 8)
+		{
+			Log_Printf(LOG_ERROR, "Note: This version of Hatari has been built"
+			                      " _without_ support for large files,\n"
+			                      "      so you can not use HD images > 2 GB.\n");
+		}
+		return -EFBIG;
+	}
+	if (filesize == 0)
+	{
+		Log_AlertDlg(LOG_ERROR, "Can not use HD image file\n'%s'\n"
+		                        "since the file is empty.",
+		             shortname);
+		return -EINVAL;
+	}
+	if ((filesize & 0x1ff) != 0)
+	{
+		Log_AlertDlg(LOG_ERROR, "Can not use the hard disk image file\n"
+		                        "'%s'\nsince its size is not a multiple"
+		                        " of 512.",
+		            shortname);
+		return -EINVAL;
+	}
+
+	return filesize;
+}
+
 /**
  * Open a disk image file
  */
@@ -715,18 +765,10 @@ static int HDC_InitDevice(SCSI_DEV *dev, char *filename)
 	dev->enabled = false;
 	Log_Printf(LOG_INFO, "Mounting hard drive image '%s'\n", filename);
 
-	/* Check size for sanity - is the length a multiple of 512? */
-	filesize = File_Length(filename);
+	/* Check size for sanity */
+	filesize = HDC_CheckAndGetSize(filename);
 	if (filesize < 0)
-	{
-		Log_Printf(LOG_ERROR, "ERROR: unable to access/get HD file size!\n");
-		return -EINVAL;
-	}
-	if (filesize <= 0 || (filesize & 0x1ff) != 0)
-	{
-		Log_Printf(LOG_ERROR, "ERROR: HD file has strange size!\n");
-		return -EINVAL;
-	}
+		return filesize;
 
 	fp = fopen(filename, "rb+");
 	if (fp == NULL)
@@ -811,10 +853,7 @@ void HDC_UnInit(void)
 {
 	int i;
 
-	if (!bAcsiEmuOn)
-		return;
-
-	for (i = 0; i < MAX_ACSI_DEVS; i++)
+	for (i = 0; bAcsiEmuOn && i < MAX_ACSI_DEVS; i++)
 	{
 		if (!AcsiBus.devs[i].enabled)
 			continue;
@@ -840,7 +879,8 @@ void HDC_UnInit(void)
 	ScsiBus.resp = NULL;
 #endif
 
-	nNumDrives -= nAcsiPartitions;
+	if (bAcsiEmuOn)
+		nNumDrives -= nAcsiPartitions;
 	nAcsiPartitions = 0;
 	bAcsiEmuOn = false;
 }
@@ -852,7 +892,7 @@ void HDC_UnInit(void)
  */
 void HDC_ResetCommandStatus(void)
 {
-	if (ConfigureParams.System.nMachineType != MACHINE_FALCON)
+	if (!Config_IsMachineFalcon())
 		AcsiBus.returnCode = 0;
 }
 
@@ -1089,7 +1129,7 @@ void HDC_WriteCommandByte(int addr, Uint8 byte)
 {
 	// fprintf(stderr, "HDC: Write cmd byte addr=%i, byte=%02x\n", addr, byte);
 
-	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+	if (Config_IsMachineFalcon())
 		Ncr5380_WriteByte(addr, byte);
 	else if (bAcsiEmuOn)
 		Acsi_WriteCommandByte(addr, byte);
@@ -1101,7 +1141,7 @@ void HDC_WriteCommandByte(int addr, Uint8 byte)
 short int HDC_ReadCommandByte(int addr)
 {
 	Uint16 ret;
-	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+	if (Config_IsMachineFalcon())
 		ret = Ncr5380_ReadByte(addr);
 	else
 		ret = AcsiBus.returnCode;	/* ACSI status */
