@@ -9,10 +9,14 @@
 #define HATARI_FILE_H
 
 #include "config.h"
+#include <sys/types.h>		/* Needed for off_t */
 
 #ifndef HAVE_FSEEKO
 #define fseeko fseek
 #endif
+#ifndef HAVE_FTELLO
+#define ftello ftell
+#endif
 
 extern void File_CleanFileName(char *pszFileName);
 extern void File_AddSlashToEndFileName(char *pszFileName);
