@@ -15,6 +15,8 @@ extern int ConOutDevice;
 #define CONOUT_DEVICE_NONE 127 /* valid ones are 0-7 */
 
 extern Uint32 Opt_GetNoParachuteFlag(void);
+extern bool Opt_IsAtariProgram(const char *path);
+extern int Opt_ValueAlignMinMax(int value, int align, int min, int max);
 extern bool Opt_ParseParameters(int argc, const char * const argv[]);
 extern char *Opt_MatchOption(const char *text, int state);
 
