@@ -37,6 +37,7 @@ const char IoMem_fileid[] = "Hatari ioMem.c : " __DATE__ " " __TIME__;
 #include "memorySnapShot.h"
 #include "m68000.h"
 #include "sysdeps.h"
+#include "newcpu.h"
 
 
 static void (*pInterceptReadTable[0x8000])(void);     /* Table with read access handlers */
@@ -47,6 +48,32 @@ Uint32 IoAccessBaseAddress;                           /* Stores the base address
 Uint32 IoAccessCurrentAddress;                        /* Current byte address while handling WORD and LONG accesses */
 static int nBusErrorAccesses;                         /* Needed to count bus error accesses */
 
+
+/*
+  Heuristics for better cycle accuracy when "cycle exact mode" is not used
+
+  Some instructions can do several IO accesses that will be seen as several independent accesses,
+  instead of one whole word or long word access as in the size of the instruction.
+  For example :
+    - movep.w and move.l will do 2 or 4 BYTE accesses (and not 1 WORD or LONG WORD access)
+    - move.l will do 2 WORD accesses (and not 1 LONG WORD, because ST's bus is 16 bit)
+
+  So, when a BYTE access is made, we need to know if it comes from an instruction where size=byte
+  or if it comes from a word or long word instruction.
+
+  In order to emulate correct read/write cycles when IO regs are accessed this way, we need to
+  keep track of how many accesses were made by the same instruction.
+  This will be used when CPU runs in "prefetch mode" and we try to approximate internal cycles
+  (see cycles.c for heuristics using this).
+
+  When CPU runs in "cycle exact mode", this is not used because the internal cycles will be computed
+  precisely at the CPU emulation level.
+*/
+static Uint64	IoAccessInstrPrevClock;
+int		IoAccessInstrCount;			/* Number of the accesses made in the current instruction (1..4) */
+							/* 0 means no multiple accesses in the current instruction */
+
+
 /* Falcon bus mode (Falcon STe compatible bus or Falcon only bus) */
 static enum {
 	STE_BUS_COMPATIBLE,
@@ -87,7 +114,115 @@ static void IoMem_SetBusErrorRegion(Uint32 startaddr, Uint32 endaddr)
 }
 
 
-/*-----------------------------------------------------------------------*/
+/**
+ * Fill a region with void handlers.
+ */
+static void IoMem_SetVoidRegion(Uint32 startaddr, Uint32 endaddr)
+{
+	Uint32 addr;
+
+	for (addr = startaddr; addr <= endaddr; addr++)
+	{
+		pInterceptReadTable[addr - 0xff8000] = IoMem_VoidRead;
+		pInterceptWriteTable[addr - 0xff8000] = IoMem_VoidWrite;
+	}
+}
+
+
+/**
+ * Normal ST has two address which don't generate a bus error when
+ * compared to the Mega-ST. Mark them as void handlers here.
+ */
+static void IoMem_FixVoidAccessForST(void)
+{
+	IoMem_SetVoidRegion(0xff820f, 0xff820f);
+	IoMem_SetVoidRegion(0xff860f, 0xff860f);
+}
+
+/**
+ * Mega-ST has slightly different behavior with bus errors compared to
+ * the normal ST. Here we fix up the table accordingly.
+ */
+static void IoMem_FixVoidAccessForMegaST(void)
+{
+	int i;
+	Uint32 no_be_addrs[] =
+	{
+		0xff8200, 0xff8202, 0xff8204, 0xff8206, 0xff8208,
+		0xff820c, 0xff8608, 0xff860a, 0xff860c, 0
+	};
+	Uint32 no_be_regions[][2] =
+	{
+		{ 0xff8000, 0xff8000 },
+		{ 0xff8002, 0xff800d },
+		{ 0xff8a3e, 0xff8a3f },
+		{ 0, 0 }
+	};
+
+	for (i = 0; no_be_addrs[i] != 0; i++)
+	{
+		IoMem_SetVoidRegion(no_be_addrs[i], no_be_addrs[i]);
+	}
+	for (i = 0; no_be_regions[i][0] != 0; i++)
+	{
+		IoMem_SetVoidRegion(no_be_regions[i][0], no_be_regions[i][1]);
+	}
+}
+
+
+/**
+ * Fix up the IO memory access table for the Mega STE.
+ */
+static void IoMem_FixAccessForMegaSTE(void)
+{
+	int addr;
+
+	/* Mega-STE has an additional Cache/CPU control register compared to the normal STE. */
+	pInterceptReadTable[0xff8e21 - 0xff8000] = IoMem_ReadWithoutInterception;
+	pInterceptWriteTable[0xff8e21 - 0xff8000] = IoMemTabMegaSTE_CacheCpuCtrl_WriteByte;
+
+	/* VME bus - we don't support it yet, but TOS uses FF8E09 to detect the Mega-STE */
+	for (addr = 0xff8e01; addr <= 0xff8e0f; addr += 2)
+	{
+		pInterceptReadTable[addr - 0xff8000] = IoMem_ReadWithoutInterception;
+		pInterceptWriteTable[addr - 0xff8000] = IoMem_WriteWithoutInterception;
+	}
+}
+
+
+/**
+ * Fix up table for Falcon in STE compatible bus mode (i.e. less bus errors)
+ */
+static void IoMem_FixVoidAccessForCompatibleFalcon(void)
+{
+	int i;
+	Uint32 no_be_regions[][2] =
+	{
+		{ 0xff8002, 0xff8005 },
+		{ 0xff8008, 0xff800b },
+		{ 0xff800e, 0xff805f },
+		{ 0xff8064, 0xff81ff },
+		{ 0xff82c4, 0xff83ff },
+		{ 0xff8804, 0xff88ff },
+		{ 0xff8964, 0xff896f },
+		{ 0xff8c00, 0xff8c7f },
+		{ 0xff8c88, 0xff8cff },
+		{ 0xff9000, 0xff91ff },
+		{ 0xff9204, 0xff920f },
+		{ 0xff9218, 0xff921f },
+		{ 0xff9224, 0xff97ff },
+		{ 0xff9c00, 0xff9fff },
+		{ 0xffa200, 0xffa207 },
+		{ 0, 0 }
+	};
+
+	for (i = 0; no_be_regions[i][0] != 0; i++)
+	{
+		IoMem_SetVoidRegion(no_be_regions[i][0], no_be_regions[i][1]);
+	}
+}
+
+
 /**
  * Create 'intercept' tables for hardware address access. Each 'intercept
  * table is a list of 0x8000 pointers to a list of functions to call when
@@ -102,50 +237,33 @@ void IoMem_Init(void)
 	/* Set default IO access handler (-> bus error) */
 	IoMem_SetBusErrorRegion(0xff8000, 0xffffff);
 
-
-	/* Initialize STe bus specific registers for Falcon in FALCON STe compatible bus mode */
-	if ((ConfigureParams.System.nMachineType == MACHINE_FALCON) && (falconBusMode == STE_BUS_COMPATIBLE)) {
-		for (addr = 0xff8000; addr < 0xffd426; addr++)
-		{
-			if ( ((addr >= 0xff8002) && (addr < 0xff8006)) ||
-				 ((addr >= 0xff8008) && (addr < 0xff800c)) ||
-				 ((addr >= 0xff800e) && (addr < 0xff8060)) ||
-				 ((addr >= 0xff8064) && (addr < 0xff8200)) ||
-				 ((addr >= 0xff82c4) && (addr < 0xff8400)) ||
-				  (addr == 0xff8560)                       ||
-				  (addr == 0xff8564)                       ||
-				 ((addr >= 0xff8804) && (addr < 0xff8900)) ||
-				 ((addr >= 0xff8964) && (addr < 0xff8970)) ||
-				 ((addr >= 0xff8c00) && (addr < 0xff8c80)) ||
-				 ((addr >= 0xff8c88) && (addr < 0xff8d00)) ||
-				 ((addr >= 0xff9000) && (addr < 0xff9200)) ||
-				 ((addr >= 0xff9204) && (addr < 0xff9206)) ||
-				 ((addr >= 0xff9207) && (addr < 0xff9210)) ||
-				 ((addr >= 0xff9218) && (addr < 0xff9220)) ||
-				 ((addr >= 0xff9224) && (addr < 0xff9800)) ||
-				 ((addr >= 0xff9c00) && (addr < 0xffa000)) ||
-				 ((addr >= 0xffa200) && (addr < 0xffa208)) ||
-				  (addr == 0xffc020)                       ||
-				  (addr == 0xffc021)                       ||
-				  (addr == 0xffd020)                       ||
-				  (addr == 0xffd021)                       ||
-				  (addr == 0xffd420)                       ||
-				  (addr == 0xffd421)                       ||
-				  (addr == 0xffd425)
-			  ) {
-					pInterceptReadTable[addr - 0xff8000] = IoMem_VoidRead;      /* For 'read' */
-					pInterceptWriteTable[addr - 0xff8000] = IoMem_VoidWrite;    /* and 'write' */
- 			    }
-		}
-	}
-
 	switch (ConfigureParams.System.nMachineType)
 	{
-		case MACHINE_ST:  pInterceptAccessFuncs = IoMemTable_ST; break;
-		case MACHINE_STE: pInterceptAccessFuncs = IoMemTable_STE; break;
-		case MACHINE_TT: pInterceptAccessFuncs = IoMemTable_TT; break;
-		case MACHINE_FALCON: pInterceptAccessFuncs = IoMemTable_Falcon; break;
-		default: abort(); /* bug */
+	 case MACHINE_ST:
+		IoMem_FixVoidAccessForST();
+		pInterceptAccessFuncs = IoMemTable_ST;
+		break;
+	 case MACHINE_MEGA_ST:
+		IoMem_FixVoidAccessForMegaST();
+		pInterceptAccessFuncs = IoMemTable_ST;
+		break;
+	 case MACHINE_STE:
+		pInterceptAccessFuncs = IoMemTable_STE;
+		break;
+	 case MACHINE_MEGA_STE:
+		IoMem_FixAccessForMegaSTE();
+		pInterceptAccessFuncs = IoMemTable_STE;
+		break;
+	 case MACHINE_TT:
+		pInterceptAccessFuncs = IoMemTable_TT;
+		break;
+	 case MACHINE_FALCON:
+		if (falconBusMode == STE_BUS_COMPATIBLE)
+			IoMem_FixVoidAccessForCompatibleFalcon();
+		pInterceptAccessFuncs = IoMemTable_Falcon;
+		break;
+	 default:
+		abort(); /* bug */
 	}
 
 	/* Now set the correct handlers */
@@ -171,7 +289,7 @@ void IoMem_Init(void)
 	}
 
 	/* Set registers for Falcon DSP emulation */
-	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+	if (Config_IsMachineFalcon())
 	{
 		switch (ConfigureParams.System.nDSPType)
 		{
@@ -198,8 +316,9 @@ void IoMem_Init(void)
 		IoMem_SetBusErrorRegion(0xff8a00, 0xff8a3f);
 	}
 
-	/* Disable real time clock? */
-	if (!ConfigureParams.System.bRealTimeClock)
+	/* Disable real time clock on non-Mega machines */
+	if (ConfigureParams.System.nMachineType == MACHINE_ST
+	    || ConfigureParams.System.nMachineType == MACHINE_STE)
 	{
 		for (addr = 0xfffc21; addr <= 0xfffc3f; addr++)
 		{
@@ -209,7 +328,7 @@ void IoMem_Init(void)
 	}
 
 	/* Initialize PSG shadow registers for ST, STe, TT machines */
-	if (ConfigureParams.System.nMachineType != MACHINE_FALCON)
+	if (!Config_IsMachineFalcon())
 	{
 		for (addr = 0xff8804; addr < 0xff8900; addr++)
 		{
@@ -258,10 +377,22 @@ void IoMem_UnInit(void)
 /**
  * Handle byte read access from IO memory.
  */
-uae_u32 IoMem_bget(uaecptr addr)
+uae_u32 REGPARAM3 IoMem_bget(uaecptr addr)
 {
 	Uint8 val;
 
+	/* Check if access is made by a new instruction or by the same instruction doing multiple byte accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( table68k[ M68000_CurrentOpcode ].size == 0 )
+			IoAccessInstrCount = 0;		/* Instruction size is byte : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
 	if (addr < 0xff8000 || !regs.s)
@@ -297,11 +428,24 @@ uae_u32 IoMem_bget(uaecptr addr)
 /**
  * Handle word read access from IO memory.
  */
-uae_u32 IoMem_wget(uaecptr addr)
+uae_u32 REGPARAM3 IoMem_wget(uaecptr addr)
 {
 	Uint32 idx;
 	Uint16 val;
 
+	/* Check if access is made by a new instruction or by the same instruction doing multiple word accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( ( table68k[ M68000_CurrentOpcode ].size == 1 )
+		  && ( OpcodeFamily != i_MVMEL ) && ( OpcodeFamily != i_MVMLE ) )
+			IoAccessInstrCount = 0;		/* Instruction size is word and not a movem : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access of a long or movem.w */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
 	if (addr < 0xff8000 || !regs.s)
@@ -349,12 +493,24 @@ uae_u32 IoMem_wget(uaecptr addr)
 /**
  * Handle long-word read access from IO memory.
  */
-uae_u32 IoMem_lget(uaecptr addr)
+uae_u32 REGPARAM3 IoMem_lget(uaecptr addr)
 {
 	Uint32 idx;
 	Uint32 val;
 	int n;
 
+	/* Check if access is made by a new instruction or by the same instruction doing multiple long accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( ( OpcodeFamily != i_MVMEL ) && ( OpcodeFamily != i_MVMLE ) )
+			IoAccessInstrCount = 0;		/* Instruction is not a movem : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access of a movem.l */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
 	if (addr < 0xff8000 || !regs.s)
@@ -405,11 +561,23 @@ uae_u32 IoMem_lget(uaecptr addr)
 /**
  * Handle byte write access to IO memory.
  */
-void IoMem_bput(uaecptr addr, uae_u32 val)
+void REGPARAM3 IoMem_bput(uaecptr addr, uae_u32 val)
 {
+	/* Check if access is made by a new instruction or by the same instruction doing multiple byte accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( table68k[ M68000_CurrentOpcode ].size == 0 )
+			IoAccessInstrCount = 0;		/* Instruction size is byte : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	LOG_TRACE(TRACE_IOMEM_WR, "IO write.b $%06x = $%02x pc=%x\n", addr, val&0x0ff, M68000_GetPC());
+	LOG_TRACE(TRACE_IOMEM_WR, "IO write.b $%06x = $%02x pc=%x\n", addr, val&0xff, M68000_GetPC());
 
 	if (addr < 0xff8000 || !regs.s)
 	{
@@ -439,13 +607,26 @@ void IoMem_bput(uaecptr addr, uae_u32 val)
 /**
  * Handle word write access to IO memory.
  */
-void IoMem_wput(uaecptr addr, uae_u32 val)
+void REGPARAM3 IoMem_wput(uaecptr addr, uae_u32 val)
 {
 	Uint32 idx;
 
+	/* Check if access is made by a new instruction or by the same instruction doing multiple word accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( ( table68k[ M68000_CurrentOpcode ].size == 1 )
+		  && ( OpcodeFamily != i_MVMEL ) && ( OpcodeFamily != i_MVMLE ) )
+			IoAccessInstrCount = 0;		/* Instruction size is word and not a movem : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access of a long or movem.w */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	LOG_TRACE(TRACE_IOMEM_WR, "IO write.w $%06x = $%04x pc=%x\n", addr, val&0x0ffff, M68000_GetPC());
+	LOG_TRACE(TRACE_IOMEM_WR, "IO write.w $%06x = $%04x pc=%x\n", addr, val&0xffff, M68000_GetPC());
 
 	if (addr < 0x00ff8000 || !regs.s)
 	{
@@ -487,11 +668,23 @@ void IoMem_wput(uaecptr addr, uae_u32 val)
 /**
  * Handle long-word write access to IO memory.
  */
-void IoMem_lput(uaecptr addr, uae_u32 val)
+void REGPARAM3 IoMem_lput(uaecptr addr, uae_u32 val)
 {
 	Uint32 idx;
 	int n;
 
+	/* Check if access is made by a new instruction or by the same instruction doing multiple long accesses */
+	if ( IoAccessInstrPrevClock == CyclesGlobalClockCounter )
+		IoAccessInstrCount++;			/* Same instruction, increase access count */
+	else
+	{
+		IoAccessInstrPrevClock = CyclesGlobalClockCounter;
+		if ( ( OpcodeFamily != i_MVMEL ) && ( OpcodeFamily != i_MVMLE ) )
+			IoAccessInstrCount = 0;		/* Instruction is not a movem : no multiple accesses */
+		else
+			IoAccessInstrCount = 1;		/* 1st access of a movem.l */
+	}
+
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
 	LOG_TRACE(TRACE_IOMEM_WR, "IO write.l $%06x = $%08x pc=%x\n", addr, val, M68000_GetPC());
