@@ -67,6 +67,9 @@
 /* 2015/02/05	[NP]	For the new WinUAE's cpu, don't use ExceptionSource anymore when calling	*/
 /*			Exception().									*/
 /* 2015/02/11	[NP]	Replace BusErrorPC by regs.instruction_pc, to get similar code to WinUAE's cpu  */
+/* 2015/10/08	[NP]	Add M68000_AddCycles_CE() to handle cycles when running with WinUAE's cpu in	*/
+/*			'cycle exact' mode. In that case, instruction pairing don't have to be handled	*/
+/*			with some tables/heuristics anymore.						*/
 
 
 const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
@@ -83,6 +86,7 @@ const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 #include "savestate.h"
 #include "stMemory.h"
 #include "tos.h"
+#include "falcon/crossbar.h"
 
 #if ENABLE_DSP_EMU
 #include "dsp.h"
@@ -95,10 +99,10 @@ const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 /* information about current CPU instruction */
 cpu_instruction_t CpuInstruction;
 
-Uint32 BusErrorAddress;         /* Stores the offending address for bus-/address errors */
-bool bBusErrorReadWrite;        /* 0 for write error, 1 for read error */
-int nCpuFreqShift;              /* Used to emulate higher CPU frequencies: 0=8MHz, 1=16MHz, 2=32Mhz */
-int nWaitStateCycles;           /* Used to emulate the wait state cycles of certain IO registers */
+Uint32 BusErrorAddress;		/* Stores the offending address for bus-/address errors */
+bool bBusErrorReadWrite;	/* 0 for write error, 1 for read error */
+int nCpuFreqShift;		/* Used to emulate higher CPU frequencies: 0=8MHz, 1=16MHz, 2=32Mhz */
+int WaitStateCycles = 0;	/* Used to emulate the wait state cycles of certain IO registers */
 int BusMode = BUS_MODE_CPU;	/* Used to tell which part is owning the bus (cpu, blitter, ...) */
 bool CPU_IACK = false;		/* Set to true during an exception when getting the interrupt's vector number */
 
@@ -289,6 +293,7 @@ void M68000_Start(void)
  *	cpu_level : not used anymore
  *	cpu_compatible : 0/false (no prefetch for 68000/20/30)  1/true (prefetch opcode for 68000/20/30)
  *	cpu_cycle_exact : 0/false   1/true (most accurate, implies cpu_compatible)
+ *	cpu_memory_cycle_exact : 0/false   1/true (most accurate, implies cpu_compatible)
  *	address_space_24 : 1 (68000/10 and 68030 LC for Falcon), 0 (68020/30/40/60)
  *	fpu_model : 0, 68881 (external), 68882 (external), 68040 (cpu) , 68060 (cpu)
  *	fpu_strict : true/false (more accurate rounding)
@@ -300,9 +305,12 @@ void M68000_Start(void)
  *	cpu_clock_multiplier : used to speed up/slow down clock by multiple of 2 in CE mode. In Hatari
  *			we use nCpuFreqShift, so this should always be set to 2<<8 = 512 to get the same
  *			cpucycleunit as in non CE mode.
+ *	cachesize : size of cache in MB when using JIT. Not used in Hatari at the moment, set it to 0
  */
 void M68000_CheckCpuSettings(void)
 {
+	int	Freq_old = nCpuFreqShift;
+
 	if (ConfigureParams.System.nCpuFreq < 12)
 	{
 		ConfigureParams.System.nCpuFreq = 8;
@@ -318,6 +326,10 @@ void M68000_CheckCpuSettings(void)
 		ConfigureParams.System.nCpuFreq = 16;
 		nCpuFreqShift = 1;
 	}
+
+	if ( Freq_old != nCpuFreqShift )
+		M68000_ChangeCpuFreq();
+
 	changed_prefs.cpu_level = ConfigureParams.System.nCpuLevel;
 	changed_prefs.cpu_compatible = ConfigureParams.System.bCompatibleCpu;
 
@@ -337,6 +349,7 @@ void M68000_CheckCpuSettings(void)
 
 	changed_prefs.address_space_24 = ConfigureParams.System.bAddressSpace24;
 	changed_prefs.cpu_cycle_exact = ConfigureParams.System.bCycleExactCpu;
+	changed_prefs.cpu_memory_cycle_exact = ConfigureParams.System.bCycleExactCpu;
 	changed_prefs.fpu_model = ConfigureParams.System.n_FPUType;
 	changed_prefs.fpu_strict = ConfigureParams.System.bCompatibleFPU;
 
@@ -351,7 +364,12 @@ void M68000_CheckCpuSettings(void)
 	currprefs.m68k_speed = changed_prefs.m68k_speed = 0;
 	currprefs.cpu_clock_multiplier = changed_prefs.cpu_clock_multiplier = 2 << 8;
 
+	/* We don't use JIT */
+	currprefs.cachesize = changed_prefs.cachesize = 0;
 #else
+	if (ConfigureParams.System.nCpuLevel > 4)
+		ConfigureParams.System.nCpuLevel = 4;
+
 	changed_prefs.cpu_cycle_exact = 0;				/* With old UAE CPU, cycle_exact is always false */
 #endif
 
@@ -557,10 +575,11 @@ void M68000_Exception(Uint32 ExceptionNr , int ExceptionSource)
 	if ( ExceptionNr > 24 && ExceptionNr < 32 )		/* Level 1-7 interrupts */
 	{
 		/* In our case, this part is called for HBL, VBL and MFP/DSP interrupts */
-		/* (see intlev() and do_specialties() in UAE CPU core) */
-		int intnr = ExceptionNr - 24;
-		pendingInterrupts |= (1 << intnr);
-		doint();
+		/* For WinUAE CPU, we must call M68000_Update_intlev after changing pendingInterrupts */
+		/* (in order to call doint() and to update regs.ipl with regs.ipl_pin, else */
+		/* the exception might be delayed by one instruction in do_specialties()) */
+		pendingInterrupts |= (1 << ( ExceptionNr - 24 ));
+		M68000_Update_intlev();
 	}
 
 	else							/* direct CPU exceptions */
@@ -620,17 +639,31 @@ void	M68000_Update_intlev ( void )
 
 /*-----------------------------------------------------------------------*/
 /**
- * There seem to be wait states when a program accesses certain hardware
- * registers on the ST. Use this function to simulate these wait states.
+ * There are some wait states when accessing certain hardware registers on the ST.
+ * This function simulates these wait states and add the corresponding cycles.
+ *
  * [NP] with some instructions like CLR, we have a read then a write at the
  * same location, so we may have 2 wait states (read and write) to add
- * (nWaitStateCycles should be reset to 0 after the cycles were added).
+ * (WaitStateCycles should be reset to 0 after all the cycles were added
+ * in run_xx() in newcpu.c).
+ *
+ * - When CPU runs in cycle exact mode, wait states are added immediately.
+ * - For other less precise modes, all the wait states are cumulated and added
+ *   after the instruction was processed.
  */
-void M68000_WaitState(int nCycles)
+void M68000_WaitState(int WaitCycles)
 {
-	M68000_SetSpecial(SPCFLAG_EXTRA_CYCLES);
+#ifndef WINUAE_FOR_HATARI
+	WaitStateCycles += WaitCycles;				/* Cumulate all the wait states for this instruction */
 
-	nWaitStateCycles += nCycles;	/* add all the wait states for this instruction */
+#else
+	if ( ConfigureParams.System.bCycleExactCpu )
+		currcycle += ( WaitCycles * CYCLE_UNIT / 2 );	/* Add wait states immediately to the CE cycles counter */
+	else
+	{
+		WaitStateCycles += WaitCycles;			/* Cumulate all the wait states for this instruction */
+	}
+#endif
 }
 
 
@@ -656,6 +689,47 @@ int	M68000_WaitEClock ( void )
 
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Some hardware registers can only be accessed on a 4 cycles boundary
+ * (shifter color regs and shifter res reg).
+ * An extra delay should be added when needed if current cycle
+ * count is not multiple of 4.
+ */
+static void	M68000_SyncCpuBus ( bool read )
+{
+	Uint64	Cycles;
+	int	CyclesToNextBus;
+
+	if ( read )
+		Cycles = Cycles_GetClockCounterOnReadAccess();
+	else
+		Cycles = Cycles_GetClockCounterOnWriteAccess();
+
+	CyclesToNextBus = Cycles & 3;
+//fprintf ( stderr , "sync bus %lld %d\n" , Cycles, CyclesToNextBus );
+	if ( CyclesToNextBus != 0 )
+	{
+//fprintf ( stderr , "sync bus wait %lld %d\n" ,Cycles, 4-CyclesToNextBus );
+		M68000_WaitState ( 4 - CyclesToNextBus );
+	}
+}
+
+
+void	M68000_SyncCpuBus_OnReadAccess ( void )
+{
+	M68000_SyncCpuBus ( true );
+}
+
+
+void	M68000_SyncCpuBus_OnWriteAccess ( void )
+{
+	M68000_SyncCpuBus ( false );
+}
+
+
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * In case we modified the memory by accessing it directly (and bypassing
@@ -693,3 +767,53 @@ void	M68000_Flush_Data_Cache ( uaecptr addr , int size )
 
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * On real STF/STE hardware, DMA accesses are restricted to 4 MB (video addresses,
+ * FDC, STE DMA sound) in the range 0 - $3fffff (22 bits of address)
+ * When STF/STE are expanded beyond 4 MB, some special '_FRB' cookies variables
+ * need to be set in TOS to allocate an intermediate buffer in lower 4 MB
+ * that will be used to transfer data in RAM between 4 MB and 16 MB.
+ * This buffer is needed because real HW can't access RAM beyond 4 MB.
+ *
+ * In Hatari, we allow DMA addresses to use 24 bits when RAM size is set
+ * to 8 or 16 MB. This way any program / TOS version can make use of extra
+ * RAM beyond 4 MB without requiring an intermediate buffer.
+ *
+ * But it should be noted that programs using 24 bits of DMA addresses would
+ * not work on real HW ; this is just to make RAM expansion more transparent
+ * under emulation.
+ *
+ * We return a mask for bits 16-23 :
+ *  - 0x3f for compatibility with real HW and limit of 4 MB
+ *  - 0xff to allow DMA addresses beyond 4 MB (or for Falcon / TT)
+ */
+int	DMA_MaskAddressHigh ( void )
+{
+	if (Config_IsMachineTT() || Config_IsMachineFalcon())
+		return 0xff;					/* Falcon / TT can access 24 bits with DMA */
+
+	else if ( ConfigureParams.Memory.nMemorySize > 4 )	/* ST/STE with more than 4 MB */
+		return 0xff;					/* Allow 'fake' 24 bits for DMA */
+
+	else							/* ST/STE with <= 4 MB */
+		return 0x3f;					/* Limit DMA range to 22 bits (same as real HW) */
+}
+
+
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * This function should be called when the cpu freq is changed, to update
+ * other components that depend on it.
+ *
+ * For now, only Falcon mode requires some updates for the crossbar
+ */
+void	M68000_ChangeCpuFreq ( void )
+{
+	if ( Config_IsMachineFalcon() )
+	{
+		Crossbar_Recalculate_Clocks_Cycles();
+	}
+}
