@@ -57,8 +57,8 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 
 #include "hatari-glue.h"
 
-#include "falcon/hostscreen.h"
 #include "falcon/dsp.h"
+#include "falcon/videl.h"
 
 #if HAVE_GETTIMEOFDAY
 #include <sys/time.h>
@@ -107,7 +107,6 @@ static Uint32 Main_GetTicks(void)
 	return ticks_to_msec * fields.tms_utime;
 }
 #else
-# warning "times() function missing, using inaccurate SDL_GetTicks() instead."
 # define Main_GetTicks SDL_GetTicks
 #endif
 
@@ -513,11 +512,18 @@ void Main_EventHandler(void)
 		}
 		switch (event.type)
 		{
-
 		 case SDL_QUIT:
 			Main_RequestQuit(0);
 			break;
-			
+
+		 case SDL_KEYDOWN:
+			Keymap_KeyDown(&event.key.keysym);
+			break;
+
+		 case SDL_KEYUP:
+			Keymap_KeyUp(&event.key.keysym);
+			break;
+
 		 case SDL_MOUSEMOTION:               /* Read/Update internal mouse position */
 			Main_HandleMouseMotion(&event);
 			bContinueProcessing = true;
@@ -575,15 +581,43 @@ void Main_EventHandler(void)
 #endif
 			break;
 
-		 case SDL_KEYDOWN:
-			Keymap_KeyDown(&event.key.keysym);
+#if WITH_SDL2
+		 case SDL_MOUSEWHEEL:
+			/* Simulate cursor keys on mouse wheel events */
+			if (event.wheel.x > 0)
+			{
+				IKBD_PressSTKey(0x4d, true);
+				IKBD_PressSTKey(0x4d, false);
+			}
+			else if (event.wheel.x < 0)
+			{
+				IKBD_PressSTKey(0x4b, true);
+				IKBD_PressSTKey(0x4b, false);
+			}
+			if (event.wheel.y < 0)
+			{
+				IKBD_PressSTKey(0x50, true);
+				IKBD_PressSTKey(0x50, false);
+			}
+			else if (event.wheel.y > 0)
+			{
+				IKBD_PressSTKey(0x48, true);
+				IKBD_PressSTKey(0x48, false);
+			}
 			break;
 
-		 case SDL_KEYUP:
-			Keymap_KeyUp(&event.key.keysym);
+		 case SDL_WINDOWEVENT:
+			if (event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED
+			    || event.window.event == SDL_WINDOWEVENT_RESTORED)
+			{
+				SDL_UpdateRect(sdlscrn, 0, 0, 0, 0);
+				break;
+			}
+			bContinueProcessing = true;
 			break;
+#endif
 
-		default:
+		 default:
 			/* don't let unknown events delay event processing */
 			bContinueProcessing = true;
 			break;
@@ -623,6 +657,7 @@ static void Main_Init_HW(void)
 	Joy_Init();
 	FDC_Init();
 	STX_Init();
+	Video_InitTimings();
 }
 
 /*-----------------------------------------------------------------------*/
@@ -654,15 +689,17 @@ static void Main_Init(void)
 	}
 
 	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
+	Video_SetTimings ( ConfigureParams.System.nMachineType , ConfigureParams.System.VideoTimingMode );
+
 	Resolution_Init();
 	SDLGui_Init();
 	Printer_Init();
 	RS232_Init();
 	Midi_Init();
 	Control_CheckUpdates();       /* enable window embedding? */
+	Videl_Init();
 	Screen_Init();
 	Main_SetTitle(NULL);
-	HostScreen_Init();
 
 	ACIA_Init( ACIA_Array , MachineClocks.ACIA_Freq , MachineClocks.ACIA_Freq );
 	IKBD_Init();			/* After ACIA_Init */
@@ -727,7 +764,6 @@ static void Main_UnInit(void)
 	Audio_UnInit();
 	SDLGui_UnInit();
 	DSP_UnInit();
-	HostScreen_UnInit();
 	Screen_UnInit();
 	Exit680x0();
 
