@@ -77,6 +77,7 @@ enum {
 	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
 	OPT_ZOOM,
+	OPT_VIDEO_TIMING,
 	OPT_RESOLUTION,		/* TT/Falcon display options */
 	OPT_FORCE_MAX,
 	OPT_ASPECT,
@@ -108,7 +109,6 @@ enum {
 	OPT_DRIVEB_HEADS,
 	OPT_DISKA,
 	OPT_DISKB,
-	OPT_SLOWFLOPPY,
 	OPT_FASTFLOPPY,
 	OPT_WRITEPROT_FLOPPY,
 	OPT_HARDDRIVE,
@@ -134,7 +134,7 @@ enum {
 	OPT_CPU_CYCLE_EXACT,	/* WinUAE CPU/FPU/bus options */
 	OPT_CPU_ADDR24,
 	OPT_FPU_TYPE,
-	OPT_FPU_COMPATIBLE,
+/*	OPT_FPU_JIT_COMPAT, */
 	OPT_MMU,
 #endif
 	OPT_MACHINE,		/* system options */
@@ -142,7 +142,6 @@ enum {
 	OPT_DSP,
 	OPT_TIMERD,
 	OPT_FASTBOOT,
-	OPT_RTC,
 	OPT_MICROPHONE,		/* sound options */
 	OPT_SOUND,
 	OPT_SOUNDBUFFERSIZE,
@@ -222,7 +221,7 @@ static const opt_t HatariOptions[] = {
 	{ OPT_MAXHEIGHT, NULL, "--max-height",
 	  "<x>", "Maximum window height for borders & zooming" },
 	{ OPT_FORCEBPP, NULL, "--bpp",
-	  "<x>", "Force internal bitdepth (x = 8/15/16/32, 0=disable)" },
+	  "<x>", "Force internal bitdepth (x = 15/16/32, 0=disable)" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "ST/STE specific display" },
 	{ OPT_BORDERS, NULL, "--borders",
@@ -233,6 +232,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
 	{ OPT_ZOOM, "-z", "--zoom",
 	  "<x>", "Double small resolutions (1=no, 2=yes)" },
+	{ OPT_VIDEO_TIMING,   NULL, "--video-timing",
+	  "<x>", "Wakeup State for MMU/GLUE (x=ws1/ws2/ws3/ws4/random, default ws3)" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "TT/Falcon specific display" },
 	{ OPT_RESOLUTION, NULL, "--desktop",
@@ -310,8 +311,6 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Set disk image for floppy drive A" },
 	{ OPT_DISKB, NULL, "--disk-b",
 	  "<file>", "Set disk image for floppy drive B" },
-	{ OPT_SLOWFLOPPY,   NULL, "--slowfdc",
-	  "<bool>", "Slow down floppy disk access emulation (deprecated, use --fastfdc)" },
 	{ OPT_FASTFLOPPY,   NULL, "--fastfdc",
 	  "<bool>", "Speed up floppy disk access emulation (can break some programs)" },
 	{ OPT_WRITEPROT_FLOPPY, NULL, "--protect-floppy",
@@ -353,31 +352,33 @@ static const opt_t HatariOptions[] = {
 	{ OPT_CARTRIDGE, NULL, "--cartridge",
 	  "<file>", "Use ROM cartridge image <file>" },
 
+#if ENABLE_WINUAE_CPU
+	{ OPT_HEADER, NULL, NULL, NULL, "CPU/FPU/bus" },
+#else
 	{ OPT_HEADER, NULL, NULL, NULL, "CPU" },
+#endif
 	{ OPT_CPULEVEL,  NULL, "--cpulevel",
 	  "<x>", "Set the CPU type (x => 680x0) (EmuTOS/TOS 2.06 only!)" },
 	{ OPT_CPUCLOCK,  NULL, "--cpuclock",
 	  "<x>", "Set the CPU clock (x = 8/16/32)" },
 	{ OPT_COMPATIBLE, NULL, "--compatible",
 	  "<bool>", "Use a more compatible (but slower) 68000 CPU mode" },
-
 #if ENABLE_WINUAE_CPU
-	{ OPT_HEADER, NULL, NULL, NULL, "WinUAE CPU/FPU/bus" },
 	{ OPT_CPU_CYCLE_EXACT, NULL, "--cpu-exact",
 	  "<bool>", "Use cycle exact CPU emulation" },
 	{ OPT_CPU_ADDR24, NULL, "--addr24",
 	  "<bool>", "Use 24-bit instead of 32-bit addressing mode" },
-	{ OPT_FPU_TYPE, NULL, "--fpu-type",
+	{ OPT_FPU_TYPE, NULL, "--fpu",
 	  "<x>", "FPU type (x=none/68881/68882/internal)" },
-	{ OPT_FPU_COMPATIBLE, NULL, "--fpu-compatible",
-	  "<bool>", "Use more compatible, but slower FPU emulation" },
+	/*{ OPT_FPU_JIT_COMPAT, NULL, "--fpu-compatible",
+	  "<bool>", "Use more compatible, but slower FPU JIT emulation" },*/
 	{ OPT_MMU, NULL, "--mmu",
 	  "<bool>", "Use MMU emulation" },
 #endif
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Misc system" },
 	{ OPT_MACHINE,   NULL, "--machine",
-	  "<x>", "Select machine type (x = st/ste/tt/falcon)" },
+	  "<x>", "Select machine type (x = st/megast/ste/megaste/tt/falcon)" },
 	{ OPT_BLITTER,   NULL, "--blitter",
 	  "<bool>", "Use blitter emulation (ST only)" },
 	{ OPT_DSP,       NULL, "--dsp",
@@ -386,8 +387,6 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Patch Timer-D (about doubles ST emulation speed)" },
 	{ OPT_FASTBOOT, NULL, "--fast-boot",
 	  "<bool>", "Patch TOS and memvalid system variables for faster boot" },
-	{ OPT_RTC,    NULL, "--rtc",
-	  "<bool>", "Enable real-time clock" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Sound" },
 	{ OPT_MICROPHONE,   NULL, "--mic",
@@ -891,6 +890,8 @@ static bool Opt_HandleArgument(const char *path)
 			dir = strdup(Paths_GetWorkingDir());
 			prgname = path;
 		}
+		Log_Printf(LOG_INFO, "ARG = autostart program: %s\n", prgname);
+
 		/* after above, dir should point to valid dir,
 		 * then make sure that given program from that
 		 * dir will be started.
@@ -903,6 +904,7 @@ static bool Opt_HandleArgument(const char *path)
 
 	/* GEMDOS HDD directory (as argument, or for the Atari program)? */
 	if (File_DirExists(path)) {
+		Log_Printf(LOG_INFO, "ARG = GEMDOS HD dir: %s\n", path);
 		if (Opt_StrCpy(OPT_HARDDRIVE, false, ConfigureParams.HardDisk.szHardDiskDirectories[0],
 			       path, sizeof(ConfigureParams.HardDisk.szHardDiskDirectories[0]),
 			       &ConfigureParams.HardDisk.bUseHardDiskDirectories)
@@ -927,6 +929,7 @@ static bool Opt_HandleArgument(const char *path)
 	/* disk image? */
 	if (Floppy_SetDiskFileName(0, path, NULL))
 	{
+		Log_Printf(LOG_INFO, "ARG = floppy image: %s\n", path);
 		ConfigureParams.HardDisk.bBootFromHardDisk = false;
 		bLoadAutoSave = false;
 		return true;
@@ -1074,7 +1077,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			case 32:
 			case 16:
 			case 15:
-			case 8:
 				break;       /* supported */
 			case 24:
 				planes = 32; /* We do not support 24 bpp (yet) */
@@ -1092,7 +1094,12 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 		case OPT_RESOLUTION_ST:
+#if WITH_SDL2
+			fprintf(stderr, "The --desktop-st option is not supported in this version (with SDL2)!\n");
+			i++;
+#else
 			ok = Opt_Bool(argv[++i], OPT_RESOLUTION_ST, &ConfigureParams.Screen.bKeepResolutionST);
+#endif
 			break;
 			
 		case OPT_SPEC512:
@@ -1122,6 +1129,23 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.Screen.nMaxHeight += STATUSBAR_MAX_HEIGHT;
 			break;
 
+		case OPT_VIDEO_TIMING:
+			i += 1;
+			if (strcasecmp(argv[i], "random") == 0)
+				ConfigureParams.System.VideoTimingMode = VIDEO_TIMING_MODE_RANDOM;
+			else if (strcasecmp(argv[i], "ws1") == 0)
+				ConfigureParams.System.VideoTimingMode = VIDEO_TIMING_MODE_WS1;
+			else if (strcasecmp(argv[i], "ws2") == 0)
+				ConfigureParams.System.VideoTimingMode = VIDEO_TIMING_MODE_WS2;
+			else if (strcasecmp(argv[i], "ws3") == 0)
+				ConfigureParams.System.VideoTimingMode = VIDEO_TIMING_MODE_WS3;
+			else if (strcasecmp(argv[i], "ws4") == 0)
+				ConfigureParams.System.VideoTimingMode = VIDEO_TIMING_MODE_WS4;
+			else
+				return Opt_ShowError(OPT_VIDEO_TIMING, argv[i], "Unknown video timing mode");
+			fprintf(stderr, "\nvideo-timing : Work in progress, this option has no effect at the moment\n\n");
+			break;
+
 			/* Falcon/TT display options */
 		case OPT_RESOLUTION:
 			ok = Opt_Bool(argv[++i], OPT_RESOLUTION, &ConfigureParams.Screen.bKeepResolution);
@@ -1355,11 +1379,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_ERROR, argv[i], "Not a disk image");
 			break;
 
-		case OPT_SLOWFLOPPY:
-			i++;
-			fprintf(stderr, "\nWarning: --slowfdc is not supported anymore, use --fastfdc\n\n");
-			break;
-
 		case OPT_FASTFLOPPY:
 			ok = Opt_Bool(argv[++i], OPT_FASTFLOPPY, &ConfigureParams.DiskImage.FastFloppy);
 			break;
@@ -1413,7 +1432,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			else if (strlen(argv[i]) == 1)
 			{
-				int drive = toupper(argv[i][0]);
+				drive = toupper(argv[i][0]);
 				if (drive >= 'C' && drive <= 'Z')
 				{
 					drive = drive - 'C' + DRIVE_C;
@@ -1613,11 +1632,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			bLoadAutoSave = false;
 			break;
-
-		case OPT_FPU_COMPATIBLE:
+/*
+		case OPT_FPU_JIT_COMPAT:
 			ok = Opt_Bool(argv[++i], OPT_FPU_COMPATIBLE, &ConfigureParams.System.bCompatibleFPU);
 			break;
-
+*/
 		case OPT_MMU:
 			ok = Opt_Bool(argv[++i], OPT_MMU, &ConfigureParams.System.bMMU);
 			bLoadAutoSave = false;
@@ -1633,12 +1652,24 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				ConfigureParams.System.nCpuLevel = 0;
 				ConfigureParams.System.nCpuFreq = 8;
 			}
+			else if (strcasecmp(argv[i], "megast") == 0)
+			{
+				ConfigureParams.System.nMachineType = MACHINE_MEGA_ST;
+				ConfigureParams.System.nCpuLevel = 0;
+				ConfigureParams.System.nCpuFreq = 8;
+			}
 			else if (strcasecmp(argv[i], "ste") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_STE;
 				ConfigureParams.System.nCpuLevel = 0;
 				ConfigureParams.System.nCpuFreq = 8;
 			}
+			else if (strcasecmp(argv[i], "megaste") == 0)
+			{
+				ConfigureParams.System.nMachineType = MACHINE_MEGA_STE;
+				ConfigureParams.System.nCpuLevel = 0;
+				ConfigureParams.System.nCpuFreq = 16;
+			}
 			else if (strcasecmp(argv[i], "tt") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_TT;
@@ -1659,17 +1690,18 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_MACHINE, argv[i], "Unknown machine type");
 			}
 #if ENABLE_WINUAE_CPU
-			if (ConfigureParams.System.nMachineType == MACHINE_ST ||
-			    ConfigureParams.System.nMachineType == MACHINE_STE)
+			if (Config_IsMachineST() || Config_IsMachineSTE())
 			{
 				ConfigureParams.System.bMMU = false;
 				ConfigureParams.System.bAddressSpace24 = true;
 			}
-			if (ConfigureParams.System.nMachineType == MACHINE_TT)
+			if (Config_IsMachineTT())
 			{
 				ConfigureParams.System.bCompatibleFPU = true;
 				ConfigureParams.System.n_FPUType = FPU_68882;
-			} else {
+			}
+			else
+			{
 				ConfigureParams.System.n_FPUType = FPU_NONE;	/* TODO: or leave it as-is? */
 			}
 #endif
@@ -1691,10 +1723,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ok = Opt_Bool(argv[++i], OPT_FASTBOOT, &ConfigureParams.System.bFastBoot);
 			break;
 
-		case OPT_RTC:
-			ok = Opt_Bool(argv[++i], OPT_RTC, &ConfigureParams.System.bRealTimeClock);
-			break;
-
 		case OPT_DSP:
 			i += 1;
 			if (strcasecmp(argv[i], "none") == 0)
@@ -1971,7 +1999,7 @@ char *Opt_MatchOption(const char *text, int state)
 		i = 0;
 	}
 	/* next match */
-	while (i < ARRAYSIZE(HatariOptions)) {
+	while (i < ARRAY_SIZE(HatariOptions)) {
 		name = HatariOptions[i++].str;
 		if (name && strncasecmp(name, text, len) == 0)
 			return (strdup(name));
