@@ -17,7 +17,6 @@ const char ShortCut_fileid[] = "Hatari shortcut.c : " __DATE__ " " __TIME__;
 #include "floppy.h"
 #include "joy.h"
 #include "keymap.h"
-#include "m68000.h"
 #include "memorySnapShot.h"
 #include "reset.h"
 #include "screen.h"
@@ -30,6 +29,7 @@ const char ShortCut_fileid[] = "Hatari shortcut.c : " __DATE__ " " __TIME__;
 #include "video.h"
 #include "avi_record.h"
 #include "clocks_timings.h"
+#include "statusbar.h"
 
 static SHORTCUTKEYIDX ShortCutKey = SHORTCUT_NONE;  /* current shortcut key */
 
@@ -228,14 +228,12 @@ static void ShortCut_InsertDisk(int drive)
 	char *selname, *zip_path = NULL;
 	const char *tmpname;
 	char FileNameB[ FILENAME_MAX ];
-	char ZipPathB[ FILENAME_MAX ];
 
 	if (SDLGui_SetScreen(sdlscrn))
 		return;
 
 	/* Save current names for drive 1 before checking autoinsert */
 	strcpy ( FileNameB , ConfigureParams.DiskImage.szDiskFileName[ 1 ] );
-	strcpy ( ZipPathB , ConfigureParams.DiskImage.szDiskZipPath[ 1 ] );
 
 	if (ConfigureParams.DiskImage.szDiskFileName[drive][0])
 		tmpname = ConfigureParams.DiskImage.szDiskFileName[drive];
@@ -251,8 +249,7 @@ static void ShortCut_InsertDisk(int drive)
 		else
 			Floppy_SetDiskFileNameNone(drive);
 
-		if (zip_path)
-			free(zip_path);
+		free(zip_path);
 		free(selname);
 		
 		Floppy_InsertDiskIntoDrive(0);
@@ -279,48 +276,50 @@ void ShortCut_ActKey(void)
 	switch (ShortCutKey)
 	{
 	 case SHORTCUT_OPTIONS:
-		Dialog_DoProperty();           /* Show options dialog */
+		Dialog_DoProperty();		/* Show options dialog */
 		break;
 	 case SHORTCUT_FULLSCREEN:
-		ShortCut_FullScreen();         /* Switch between fullscreen/windowed mode */
+		ShortCut_FullScreen();		/* Switch between fullscreen/windowed mode */
 		break;
 	 case SHORTCUT_MOUSEGRAB:
-		ShortCut_MouseGrab();          /* Toggle mouse grab */
+		ShortCut_MouseGrab();		/* Toggle mouse grab */
 		break;
 	 case SHORTCUT_COLDRESET:
 		Main_UnPauseEmulation();
-		Reset_Cold();                  /* Reset emulator with 'cold' (clear all) */
+		Reset_Cold();			/* Reset emulator with 'cold' (clear all) */
+		Statusbar_UpdateInfo();		/* Some infos can change after 'reset' */
 		break;
 	 case SHORTCUT_WARMRESET:
 		Main_UnPauseEmulation();
-		Reset_Warm();                  /* Emulator 'warm' reset */
+		Reset_Warm();			/* Emulator 'warm' reset */
+		Statusbar_UpdateInfo();		/* Some infos can change after 'reset' */
 		break;
 	 case SHORTCUT_SCREENSHOT:
-		ScreenSnapShot_SaveScreen();   /* Grab screenshot */
+		ScreenSnapShot_SaveScreen();	/* Grab screenshot */
 		break;
 	 case SHORTCUT_BOSSKEY:
-		ShortCut_BossKey();            /* Boss key */
+		ShortCut_BossKey();		/* Boss key */
 		break;
-	 case SHORTCUT_CURSOREMU:          /* Toggle joystick emu on/off */
+	 case SHORTCUT_CURSOREMU:		/* Toggle joystick emu on/off */
 		Joy_ToggleCursorEmulation();
 		break;
 	 case SHORTCUT_FASTFORWARD:
-		ShortCut_FastForward();       /* Toggle Min/Max speed */
+		ShortCut_FastForward();		/* Toggle Min/Max speed */
 		break;
 	 case SHORTCUT_RECANIM:
-		ShortCut_RecordAnimation();    /* Record animation */
+		ShortCut_RecordAnimation();	/* Record animation */
 		break;
 	 case SHORTCUT_RECSOUND:
-		ShortCut_RecordSound();        /* Toggle sound recording */
+		ShortCut_RecordSound();		/* Toggle sound recording */
 		break;
 	 case SHORTCUT_SOUND:
-		ShortCut_SoundOnOff();         /* Enable/disable sound */
+		ShortCut_SoundOnOff();		/* Enable/disable sound */
 		break;
 	 case SHORTCUT_DEBUG:
-		ShortCut_Debug();              /* Invoke the Debug UI */
+		ShortCut_Debug();		/* Invoke the Debug UI */
 		break;
 	 case SHORTCUT_PAUSE:
-		ShortCut_Pause();              /* Invoke Pause */
+		ShortCut_Pause();		/* Invoke Pause */
 		break;
 	 case SHORTCUT_JOY_0:
 		Joy_SwitchMode(0);
