@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # 
-# Copyright (C) 2012-2013 by Eero Tamminen <oak at helsinkinet fi>
+# Copyright (C) 2012-2015 by Eero Tamminen <oak at helsinkinet fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -107,16 +107,16 @@ class TOS:
     
     def _add_info(self):
         "add TOS version specific info of supported machines etc"
-        name, version = self.name, self.version
+        name, size, version = self.name, self.size, self.version
         
         if self.etos:
             # EmuTOS 512k, 256k and 192k versions have different machine support
-            if self.size == 512:
+            if size == 512:
                 # startup screen on falcon 14MB is really slow
                 info = (5, 10, ("st", "ste", "tt", "falcon"))
-            elif self.size == 256:
+            elif size == 256:
                 info = (2, 8, ("st", "ste", "tt"))
-            elif self.size == 192:
+            elif size == 192:
                 info = (0, 6, ("st",))
             else:
                 raise AssertionError("'%s' image size %dkB isn't valid for EmuTOS" % (name, size))
@@ -128,7 +128,8 @@ class TOS:
         elif version < 0x200:
             info = (0, 6, ("ste",))
         elif version < 0x300:
-            info = (1, 6, ("st", "ste", "tt"))
+            # are slower with VDI mode than others
+            info = (2, 8, ("st", "ste", "tt"))
         elif version < 0x400:
             # memcheck comes up fast, but boot takes time
             info = (2, 8, ("tt",))
@@ -139,7 +140,7 @@ class TOS:
             raise AssertionError("'%s' TOS version 0x%x isn't valid" % (name, version))
         
         if self.etos:
-            print "%s is EmuTOS v%x %dkB" % (name, version, self.size)
+            print "%s is EmuTOS v%x %dkB" % (name, version, size)
         else:
             print "%s is normal TOS v%x" % (name, version)
         # 0: whether / how long to wait to dismiss memory test
@@ -185,6 +186,12 @@ class TOS:
                 return False
         return True
 
+    def supports_32bit_addressing(self):
+        "whether TOS version supports 32-bit addressing"
+        if self.etos or self.version >= 0x300:
+            return True
+        return False
+
 
 # -----------------------------------------------
 def validate(args, full):
@@ -203,19 +210,20 @@ class Config:
     fast = False
     bools = []
     disks = ("floppy", "gemdos")
-    graphics = ("mono", "rgb", "vdi1")
+    graphics = ("mono", "rgb", "vga", "vdi1", "vdi4")
     machines = ("st", "ste", "tt", "falcon")
     memsizes = (0, 4, 14)
+    ttrams = (0, 32)
 
     def __init__(self, argv):
-        longopts = ["bool=", "disks=", "fast", "graphics=", "help", "machines=", "memsizes="]
+        longopts = ["bool=", "disks=", "fast", "graphics=", "help", "machines=", "memsizes=", "ttrams="]
         try:
-            opts, paths = getopt.gnu_getopt(argv[1:], "b:d:fg:hm:s:", longopts)
+            opts, paths = getopt.gnu_getopt(argv[1:], "b:d:fg:hm:s:t:", longopts)
         except getopt.GetoptError as error:
             self.usage(error)
         self.handle_options(opts)
         self.images = self.check_images(paths)
-        print "Test configuration:\n\t", self.disks, self.graphics, self.machines, self.memsizes
+        print "Test configuration:\n\t", self.disks, self.graphics, self.machines, self.memsizes, self.ttrams
 
     
     def check_images(self, paths):
@@ -254,6 +262,15 @@ class Config:
                 except ValueError:
                     self.usage("non-numeric memory sizes: %s" % arg)
                 unknown, self.memsizes = validate(args, self.all_memsizes)
+            elif opt in ("-t", "--ttrams"):
+                try:
+                    args = [int(i) for i in args]
+                except ValueError:
+                    self.usage("non-numeric TT-RAM sizes: %s" % arg)
+                for ram in args:
+                    if ram < 0 or ram > 256:
+                        self.usage("invalid TT-RAM (0-256) size: %d" % ram)
+                self.ttrams = args
             if unknown:
                 self.usage("%s are invalid values for %s" % (list(unknown), opt))
     
@@ -272,6 +289,7 @@ Options:
 \t-g, --graphics\t%s
 \t-m, --machines\t%s
 \t-s, --memsizes\t%s
+\t-t, --ttrams\t%s
 \t-b, --bool\t(extra boolean Hatari options to test)
 
 Multiple values for an option need to be comma separated. If some
@@ -282,9 +300,10 @@ For example:
 \t--disks gemdos \\
 \t--machines st,tt \\
 \t--memsizes 0,4,14 \\
+\t--ttrams 0,32 \\
 \t--graphics mono,rgb \\
 \t-bool --compatible,--rtc
-""" % (name, self.all_disks, self.all_graphics, self.all_machines, self.all_memsizes, name))
+""" % (name, self.all_disks, self.all_graphics, self.all_machines, self.all_memsizes, self.ttrams, name))
         if msg:
             print("ERROR: %s\n" % msg)
         sys.exit(1)
@@ -339,9 +358,27 @@ For example:
             return True
         return False
 
+    def valid_ttram(self, machine, tos, ttram, winuae):
+        "return whether given TT-RAM size is valid for given machine"
+        if machine in ("st", "ste"):
+            if ttram == 0:
+                return True
+        elif machine in ("tt", "falcon"):
+            if ttram == 0:
+                return True
+            if not winuae:
+                warning("TT-RAM / 32-bit addressing is supported only by Hatari WinUAE CPU core version")
+                return False
+            if ttram < 0 or ttram > 256:
+                return False
+            return tos.supports_32bit_addressing()
+        else:
+            raise AssertionError("unknown machine %s" % machine)
+        return False
+
 
 # -----------------------------------------------
-def verify_match(srcfile, dstfile):
+def verify_file_match(srcfile, dstfile):
     "return error string if given files are not identical"
     if not os.path.exists(dstfile):
         return "file '%s' missing" % dstfile
@@ -352,7 +389,7 @@ def verify_match(srcfile, dstfile):
         if line != f2.readline():
             return "file '%s' line %d doesn't match file '%s'" % (dstfile, i, srcfile)
 
-def verify_empty(srcfile):
+def verify_file_empty(srcfile):
     "return error string if given file isn't empty"
     if not os.path.exists(srcfile):
         return "file '%s' missing" % srcfile
@@ -385,6 +422,9 @@ class Tester:
         self.create_config()
         self.create_files()
         signal.signal(signal.SIGALRM, self.alarm_handler)
+        hatari = hconsole.Hatari(["--confirm-quit", "no"])
+        self.winuae = hatari.winuae
+        hatari.kill_hatari()
     
     def alarm_handler(self, signum, dummy):
         "output error if (timer) signal came before passing current test stage"
@@ -399,6 +439,7 @@ class Tester:
         # write specific configuration to:
         # - avoid user's own config
         # - get rid of the dialogs
+        # - don't warp mouse on resolution changes
         # - limit Videl zooming to same sizes as ST screen zooming
         # - get rid of statusbar and borders in TOS screenshots
         #   to make them smaller & more consistent
@@ -408,13 +449,13 @@ class Tester:
         # - disable serial in and set serial output file
         # - disable MIDI in, use MIDI out as fifo file to signify test completion
         dummy = open(self.dummycfg, "w")
-        dummy.write("[Log]\nnAlertDlgLogLevel = 0\nbConfirmQuit = FALSE\n")
-        dummy.write("[Screen]\nnMaxWidth=832\nnMaxHeight=576\nbCrop = TRUE\nbAllowOverscan=FALSE\n")
-        dummy.write("[HardDisk]\nbUseHardDiskDirectory = FALSE\n")
-        dummy.write("[Floppy]\nszDiskAFileName = blank-a.st.gz\n")
-        dummy.write("[Printer]\nbEnablePrinting = TRUE\nszPrintToFileName = %s\n" % self.printout)
-        dummy.write("[RS232]\nbEnableRS232 = TRUE\nszInFileName = \nszOutFileName = %s\n" % self.serialout)
-        dummy.write("[Midi]\nbEnableMidi = TRUE\nsMidiInFileName = \nsMidiOutFileName = %s\n" % self.fifofile)
+        dummy.write("[Log]\nnAlertDlgLogLevel = 0\nbConfirmQuit = FALSE\n\n")
+        dummy.write("[Screen]\nnMaxWidth = 832\nnMaxHeight = 576\nbCrop = TRUE\nbAllowOverscan = FALSE\nbMouseWarp = FALSE\n\n")
+        dummy.write("[HardDisk]\nbUseHardDiskDirectory = FALSE\n\n")
+        dummy.write("[Floppy]\nszDiskAFileName = blank-a.st.gz\n\n")
+        dummy.write("[Printer]\nbEnablePrinting = TRUE\nszPrintToFileName = %s\n\n" % self.printout)
+        dummy.write("[RS232]\nbEnableRS232 = TRUE\nszInFileName = \nszOutFileName = %s\n\n" % self.serialout)
+        dummy.write("[Midi]\nbEnableMidi = TRUE\nsMidiInFileName = \nsMidiOutFileName = %s\n\n" % self.fifofile)
         dummy.close()
 
     def cleanup_all_files(self):
@@ -453,19 +494,19 @@ class Tester:
         # GEMDOS operations work properly...
         ok = True
         # check file truncate
-        error = verify_empty(self.textoutput)
+        error = verify_file_empty(self.textoutput)
         if error:
             print "ERROR: file wasn't truncated:\n\t%s" % error
             os.rename(self.textoutput, "%s.%s" % (self.textoutput, identity))
             ok = False
         # check serial output
-        error = verify_match(self.textinput, self.serialout)
+        error = verify_file_match(self.textinput, self.serialout)
         if error:
             print "ERROR: serial output doesn't match input:\n\t%s" % error
             os.rename(self.serialout, "%s.%s" % (self.serialout, identity))
             ok = False
         # check printer output
-        error = verify_match(self.textinput, self.printout)
+        error = verify_file_match(self.textinput, self.printout)
         if error:
             if tos.etos or tos.version > 0x206 or (tos.version == 0x100 and memory > 1):
                 print "ERROR: printer output doesn't match input (EmuTOS, TOS v1.00 or >v2.06)\n\t%s" % error
@@ -473,7 +514,7 @@ class Tester:
                 ok = False
             else:
                 if os.path.exists(self.printout):
-                    error = verify_empty(self.printout)
+                    error = verify_file_empty(self.printout)
                     if error:
                         print "WARNING: unexpected printer output (TOS v1.02 - TOS v2.06):\n\t%s" % error
                         os.rename(self.printout, "%s.%s" % (self.printout, identity))
@@ -513,8 +554,7 @@ class Tester:
         except IOError:
             print "ERROR: fifo open IOError!"
             return None
-    
-    
+
     def test(self, identity, testargs, tos, memory):
         "run single boot test with given args and waits"
         # Hatari command line options, don't exit if Hatari exits
@@ -550,10 +590,15 @@ class Tester:
         return (init_ok, prog_ok, tests_ok, output_ok)
 
     
-    def prepare_test(self, config, tos, machine, monitor, disk, memory, extra):
+    def prepare_test(self, config, tos, machine, monitor, disk, memory, ttram, extra):
         "compose test ID and Hatari command line args, then call .test()"
-        identity = "%s-%s-%s-%s-%sM" % (tos.name, machine, monitor, disk, memory)
+        identity = "%s-%s-%s-%s-%dM-%dM" % (tos.name, machine, monitor, disk, memory, ttram)
         testargs = ["--tos", tos.path, "--machine", machine, "--memsize", str(memory)]
+        if self.winuae:
+            if ttram:
+                testargs += ["--addr24", "off", "--ttram", str(ttram)]
+            else:
+                testargs += ["--addr24", "on"]
         
         if extra:
             identity += "-%s%s" % (extra[0].replace("-", ""), extra[1])
@@ -561,7 +606,7 @@ class Tester:
 
         if monitor.startswith("vdi"):
             planes = monitor[-1]
-            testargs +=  ["--vdi-planes", planes]
+            testargs += ["--vdi-planes", planes]
             if planes == "1":
                 testargs += ["--vdi-width", "800", "--vdi-height", "600"]
             elif planes == "2":
@@ -611,17 +656,20 @@ class Tester:
                     for memory in config.memsizes:
                         if not config.valid_memsize(machine, memory):
                             continue
-                        for disk in config.disks:
-                            if not config.valid_disktype(machine, tos, disk):
+                        for ttram in config.ttrams:
+                            if not config.valid_ttram(machine, tos, ttram, self.winuae):
                                 continue
-                            if config.bools:
-                                for opt in config.bools:
-                                    for val in ('on', 'off'):
-                                        self.prepare_test(config, tos, machine, monitor, disk, memory, [opt, val])
-                                        count += 1
-                            else:
-                                self.prepare_test(config, tos, machine, monitor, disk, memory, None)
-                                count += 1
+                            for disk in config.disks:
+                                if not config.valid_disktype(machine, tos, disk):
+                                    continue
+                                if config.bools:
+                                    for opt in config.bools:
+                                        for val in ('on', 'off'):
+                                            self.prepare_test(config, tos, machine, monitor, disk, memory, ttram, [opt, val])
+                                            count += 1
+                                else:
+                                    self.prepare_test(config, tos, machine, monitor, disk, memory, ttram, None)
+                                    count += 1
             if not count:
                 warning("no matching configuration for TOS '%s'" % tos.name)
         self.cleanup_all_files()
